; TelescoPIC 1.0.1    Digital setting circles and display.
;            1.0.2    added ra/dec in one line march 2007  
; This code by A.M.Hugo 2004...2007.  Squeezes into a 16F628 PIC.
; - 16F84 VERSION requires a PC connected to calculate RA and dec from encoder values

; to do list: convert 16 bit number to decimal:   done!      
;             convert counts to ra, dec hms etc:  done!
;             LED serial shifter                  done!
;             resolve hour angle + SidT = RA      done!
;             navigation stars                    done!
;             resolve dec degrees format          done!
;             add stopwatch/lap timer             done!
;             add ra/dec on single display line   hmm...
;             fix serial code                     oops
;             add kitchen sink                    pending...

; 0 words weft, thats all folks!

; Email to:  amhugo@optusnet.com.au
; Features:
; LED display of time, siderial time, RA, Dec encoder counts etc, 3 wire interface
; Digital encoders for RA and DEC axis
; Save and read values in EEPROM
; Stopwatch / lap timer mode
; Serial data communications with your computer
; Serial routines - MAX232 compatible

; Credits:
; Ideas from all over the net: Nigel Goodwin (RS232)
; Ed ? (digital setting circles) etc, 2003
                                   
; micro   program mem    RAM  bank0  bank1  bank2    bank3    EEPROM
; 16F84   1k 0000-03FF    68  0C-4F                           64   2000-203F
; 16F628  2k 0000-07ff   224  20-7F  A0-EF  120-14F           128  2000-207F
; 16F876  8k 0000-07FF   368  20-7F  A0-EF  120-16F  1A0-1EF  256  2000-20FF
;   ...      0800-0FFF page1
;   ...      1000-17FF page2  (can you believe having this crazy segmented garbage?)
;   ...      1800-1FFF page3


; Hardware assignments

; LED port       RB7 data     shifts segment data and digit address to two 74HC595;
;                RB6 enable   latch outputs of 74HC595 shifter when shifting is done.
;                RB5 clock    shift clock, toggle for each bit.

; Encoder  DecA  RB4   inputs, Declination
;          DecB  RB3
;          HrA   RB2   Hour Angle...
;          HrB   RB1

; RS232 Receive  RB0   Serial input    via MAX232 rs232 converter
; RS232 Transmit RA0          output

; LED lamps  L1  RA1   output  seconds indicator?
;
; Pushbutton P1  RA2   inputs, mode set: display, set
;            P2  RA3   plus    increment 
;            P3  RA4   minus   decrement 
;            p4  RA5   okay    (MCLR)  PB4 input if mclr not used

; PortA  ..111100   2 outputs, 4 inputs    0x3C
; PortB  00011111   3 outputs, 5 inputs    0x1F

; Siderial clock for RA conversion of hour angle encoder
; Real time clock in case you stay out too late
; Stand alone mode. Doesn't need to be connected to a PC. (download/upload if it is)

; Timer runs from 20MHz clock - 1/5 uS  (5 ticks per uSec)
; divide by 256 with 256 scaler means an interupt every 65536/5 uS = 13.1072mSec
; An internal counter is used to check for 1 million uS (5 million counts) (every interrupt),
; if successful, seconds are updated in normal/siderial clock. This is at the
; expense of irregular seconds update, up to 13mS error.

; With the LED version, we can't send characters to the display and forget it. We need to
; 'refresh' each digit in turn every 2mS so we need a display buffer in RAM (EEPROM too slow).

; LED version, up to 12 char, needs updating 50 times per second or 1.6mS per digit
; which means 8,000 clocks approx. We could use an interupt of 8192 clocks (1.6mSec).
; We will use timer 1, and load it up with 65536-8192 every TMR1 interupt.

; Siderial timer is used to generate RA coords from encoder hour angle.
; It accurately follows the siderial rate but it is not 'correct" in an absolute sense.
; It is adjusted everytime the telescope is synched with the sky, to a value which
; makes the internal calcs work; so don't rely on it for truth.

; It would be nice to have a stopwatch or lap timer function.
; I don't want to alter the main timer code to do this.
; Timer 1, the LED display update could be adjusted from 1.6mS to another period
; such as 1.0 mS or 2.0 mS but I would like the option of keeping things as they are.
; Timer 2 is available and can be programmed to provide regular interupts which can
; be started and stopped at will without affecting the other timing code.

; Postscaler 10  Prescaler 4, period register PR2 125 gives  5000 counts/irq = 1000 Hz = 1 ms
;             5            4                      250        5000              1000      1
;            10            4                      250       10000               500 Hz = 2 mS
;             5           16                      250       20000               250 Hz = 4 mS
;            10           16                      250       40000               125 Hz = 8 mS
; In lap timer mode, a counter in KHz can be incremented each interupt.
; When a button is pressed, this counter can be copied to RAM and displayed on the LEDs.
; as   "LAP 01234.56"
; Button '+' could be "start counter"
;        '-'          "stop counter and save in EEPROM"
;        'OK'         "record lap time"
; Bank one ram could be used and perhaps part of EEPROM to record values for later download.
; we only need to store the counter bytes for each lap time.
; In the EEPROM we have 32 bytes of data plus 5 bytes per set point for Ra/Dec.
; If we limit this to 8 set points, this will leave 56 bytes free.
; If we record the time at the start (3 bytes) plus 16 records (48 bytes) there will 
; be 5 bytes remaining for ROM ID and version.
; The problem will be implementing all this and retrieving it, with 140 code bytes left!

; 2 byte counter at 100Hz = 655.36 seconds (10.9 minutes)
; 2 byte counter at 1kHz = 65.536 seconds
; 3 byte counter at 1kHz = 16,777.215 seconds (over 4.6 hours)
; 3 byte counter at 100Hz = 167,772.15 seconds (over 46 hours)

; OPERATION

; SET UP

; Connect unit to telescope. First task is to set your lattitude...
; Align telescope on meridian (vertical) so that DEC display is the same as your lattitude
; Press mode button until "Set Lat" is displayed on LED
; Press OK button to activate
; This will store the declination encoder count as the Latitude in EEPROM.
; From now on, start each session by setting the time and declination counters
; followed by synching with one alignment star to set the siderial time as follows..

; AT THE START OF EACH SESSION:
; SETTING TIMES:
;
; Press mode button until "tod hr" (Time of day, hours) is shown on LEDs
; Press '+' or '-' key to increment or decrement the hours value
; Repeat for "TOD nn" (Time of day, minutes)
; There is no need to set the Siderial time.

; SETTING THE DEC AXIS

; Align telescope on the local meridian with the tube vertical, with a level.
; Press mode button until "Set dEc" is displayed on LED
; Press OK button to activate
; This will centre RA encoder and set DEC to the Latitude stored in EEPROM
; This will give rough alignment in DEC but not RA

; ALIGN TELESCOPE   (USING ALIGNMENT STARS)         

; Move telescope so that the selected star (say, Rigel) is centred,
; Press mode button until you see "Set" followed by a starname on the display
; Press + or - to change to the desired star name. (16 nav stars available)
; Press OK button to change the RA and DEC to the star position & update encoders.

; CREATE A SET POINT   (USING ANY STAR OR OBJECT)         

; Move telescope so that RA, DEC are true to selected object (say, M42),
; Press mode button until "Set Pt" is displayed on LED
; Press + or - to change to the desired set point, 0-15
; Press OK button to save RA, DEC position in EEPROM

; Centre the same object in the telescope and execute the RECALL routine, next.

; SETTING THE RA & DEC ENCODER COUNTS FROM A SET POINT

; Move to the object used as a set point (say, M42) and centre in eyepiece or CCD
; Press mode button until "Recall " is displayed on LED
; Press + or - to change to the desired set point, 0-15
; Press OK button to synch RA and Dec counters to this object (Stored in EEPROM)
; This will recall last values for RA, DEC and set Siderial time & Encoders as needed.

; CHANGE ENCODER LIMITS TO SUIT YOUR ENCODER TYPE

; Press mode button until "RA top" is displayed on LED
; Press '+' or '-' key to increment or decrement the value & save in EEPROM

; Press mode button until "dEctop" is displayed on LED
; Press '+' or '-' key to increment or decrement the value & save in EEPROM

; Note: if the encoder limits are changed, you will need to change the corresponding
; calculation constant as well.

; EDIT A CALCULATION VARIABLE

; The EDIT mode enables you to change any constant used by the program in the data
; variables, in the range 0f 0x20 to 0x7F. This is both clever and dangerous!
; Clever because you can update any variable, dangerous because you can mess up
; everything without even trying, especially if the changes are committed to EEPROM...

; Suppose you fit a new dec encoder with 8192 counts per full revolution (360 deg).
; This will not work properly because the calculations for Dec rely on a fixed constant
; that suited the old encoder, and this constant is loaded from EEPROM when Telscopic
; is turned on, so all the dec values are now wrong. What to do...
; First calculate a new Dec calc constant, thus: 216000*65536/EncMax, or
; 216000*65536/8192 = 1728000.  In Hexadecimal this is  $1A5E00
; This value is stored at address 0x48 (low byte), 0x49 (mid byte) and 0x4A (high byte).
; Now use the EDIT mode to change these three variable addresses to your new values.

; Press mode button until "Edit" is displayed on LED along with a hex address.
; Press '+' or '-' key to increment or decrement the address until it matches the high byte.
; i.e. it will display "Edit    4A".  Press the OK button to start editing that variable.
; The display will change to " Edit 4A   15" where 15 is the current hex value at $4A.
; Press '+' or '-' key to increment or decrement the value (change it to "Edit 4A  1A")
; At this point, no change will actually occur until you press the OK button.
; (You can cancel the edit by pressing the MODE key.)
; Press OK to make the change.

; In a similar way, change the other two variables: "Edit 49  5E" and "Edit 48  00"

; At this stage, your new values can be tested, but they will only remain valid until
; TELESCOPIC is turned off. (Restarting will restore the old values from EEPROM).
; To make the changes permanent, the data bank must be saved to EEPROM.

; Note: The hour angle calculation constant is at adresses 45, 46 and 47 (hex).
; The hour angle calculation is 86400*65536/HAEncMax.

; SAVING OPERATIONAL DATA TO EEPROM

; Press MODE button until "Set EE" is displayed on LED
; Press OK button to save the data in EEPROM, overwriting previous values.

; The following information is saved in EEPROM:
; The Set Point number, Time of day, Siderial time, RA and DEC calculation constants,
; navigation star number, encoder maximum values, current encoder count, latitude count,
; edit location, RA hr-min-sec and Dec deg/min in the form dd mm.m


	LIST	p=16F628		; tell assembler what chip we are using
	include "P16F628.inc"		; include the defaults for the chip

;        LIST p=16F84
;        include "P16F84.INC"

	ERRORLEVEL	0,	-302	; suppress bank selection messages
	__config 0x3F02                 ; was F1 ; configuration settings (HS,PWRUP,WDTOFF)


porta_DDR    equ    0x3C                ; was 1C
portb_DDR    equ    0x1F
portb_DDRdebug    equ    0x07           ; use b3 & b4 for debug code
key_mask     equ    0x3C                ; was 1C
ledsec       equ       1
ledcom       equ       2
pbut1        equ       3
pbut2        equ       4
pbut3        equ       2
pbut4        equ       5

PB_MODE      equ       2
PB_PLUS      equ       3
PB_MINUS     equ       4
PB_OK        equ       5

; COMMSFLAG bit flags for serial communications
TXIP         equ       0         ; transmit in progress
RXIP         equ       1         ; receive byte in progress

; Variables etc

	cblock	0x20		  ;start of general purpose registers 16F628
                                  ; for 16F84 this is 0x0C
; RAM scratchpad - 0x20++ for P16F628
; LCD vars,  rename or use for LED 7seg display
ptr        ; equ        0x20       ; chrout pointer
lcd_tmp    ; equ        0x21       ; LCD/LED shift-out register
lcd_cnt    ; equ        0x22       ; LCD/LED shift-out counter
lcd_dly1   ; equ        0x23       ; char convert
lcd_dly2   ; equ        0x24       ; 5ms counter
lcd_buf1   ; equ        0x25       ; ra_dec work area
lcd_buf2   ; equ        0x26   
; Edit mode temp value
editval    ;            0x27       ; temp store for Edit mode
; RS232 vars:                     
trxbuff    ; equ        0x28       ; holds byte to xmit
rxbuff     ; equ        0x29       ; holds received byte 
rxbuff2    ; EQU        0x2A       ; store first if getting second byte
bitcount   ; equ        0x2B       ; bit counter for RS232
COMMSFLAG  ; EQU        0x2C       ; Flag if Tx, Rx in progress or done
                                    ; bit 0  tx in progress
                                    ; bit 1  rx incoming
                                    ; bit 7  seconds blink
; IRQ timer0 counters   do not use outside of IRQ
u0         ; equ        0x2D       ; 24 bit counter for counting 1million uS
u1         ; equ        0x2E       ;
u2         ; equ        0x2F       ;
t0         ; equ        0x30       ; 24 bit scratch pad for mega test
t1         ; equ        0x31       ;
t2         ; equ        0x32       ;
; siderial time counters  
st0        ; EQU        0x33       ; 24 bit counter for SidT
st1        ; EQU        0x34       
st2        ; EQU        0x35  
; RA, DEC encoder state vars
DECenc     ; EQU        0x36       ; encoder states and..
oldDECen   ; EQU        0x37       ; previous encoder states
RAenc      ; EQU        0x38  
oldRAenc   ; EQU        0x39  
ERRcount   ; EQU        0x3A       ; Log encoder errors (skipped counts)
HAhh       ;            0x3B       ; result, hour angle counts to hms
HAmm       ;            0x3C       ; was sidcount
HAss       ;            0x3D       ; was sidcountHi
; these next 32 vars are saved in EEPROM: 17 FOR ENCODERS, RA/DEC, 6 TIMES, 9 CALC FACTORS
pointnum    ; EQU        0x3E      ; number of setpoint to store to EEPROM
; Day timer vars
secs       ; equ        0x3F       ; time of day    - time vars touched by IRQ code
mins       ; equ        0x40       ;
hours      ; equ        0x41       ;
; Siderial timer vars
sidsec     ; equ        0x42       ; siderial time (approx)
sidmin     ; equ        0x43       ;
sidhr      ; equ        0x44       ;
; calculation constants
HAfact0    ;            0x45       ; conversion factor - RA counts to seconds 
HAfact1    ;            0x46  
HAfact2    ;            0x47  
DECfact0   ;            0x48       ; convert DEC counts to arcmin tenths
DECfact1   ;            0x49
DECfact2   ;            0x4A
decsign    ;            0x4B    declination sign 0x=pos 8x=neg (uses high bit)
; edit location for address to change in edit mode
editloc    ;            0x4C
;   Navigation star: fixed stars stored in ROM CODE (0-15)
navstar    ;            0x4D       ; Navigation star stored in ROM CODE (0-15)
; encoder data
DECmax     ; EQU        0x4E       ; DEC and RA encoder max value
DECmaxHi   ; EQU        0x4F  
; extra space in '628 not in '84
RAmax      ; EQU        0x50  
RAmaxHi    ; EQU        0x51  
; Hour angle
RAcount    ; EQU        0x52  
RAcountHi  ; EQU        0x53  
; Latitude setting count
LATcount   ; equ        0x54      ; Lattitude, in DEC encoder "counts"
LATcountHi ; equ        0x55      ; 
; (set point starts here, 5 bytes)
DECcount   ; EQU        0x56      ; DEC encoder counts
DECcountHi ; EQU        0x57
; RA and DEC storage, 16F84: these bytes set by PC rs232 comms
RAhh     ;   EQU        0x58        ; RA HOURS 0-23 
RAmm     ;   EQU        0x59        ; RAmm   0-59
RAss     ;   EQU        0x5A        ; RAss   0-59
DECdd    ;   EQU        0x5B        ; DECdd  0-90  ('decsign' bit7 set if -ve)
DECmm    ;   EQU        0x5C        ; DECMM  0-59
DECss    ;   EQU        0x5D        ; DECss  0-9 minutes decimal place (tenths)
; end of EEPROM vars
; buffer for ascii/7segment display characters
buff0    ;   EQU        0x5E        ; buffer for 7 seg display, first 4 char
buff1    ;   EQU        0x5F        ; descriptive, eg "dEc " or "rA  " etc                            
buff2    ;   EQU        0x60        ; display buffer                        
buff3    ;   EQU        0x61        ;                       
; character buffer for display of RA DEC Times and counts
buff4    ;   EQU        0x62        ; h+R+ 
buff5    ;   EQU        0x63        ; hd26 
buff6    ;   EQU        0x64        ;  d25
buff7    ;   EQU        0x65        ; m 05
buff8    ;   EQU        0x66        ; mm03
buff9    ;   EQU        0x67        ;  m06
buff10   ;   EQU        0x68        ; s.
buff11   ;   EQU        0x69        ; sd

PC_con     ; EQU        0x6A      ; PC comms connected 'N' or 'Y'
dignum     ;            0x6B      ; current display digit 0-11
keyflag    ;            0x6C      ; keys pressed logger
MSGPTR     ; EQU        0x6D      ; Tx strings from ROM code
Dmode      ;            0x6E      ; current job or display mode
XREG       ;            0x6F        general purpose counter, rs232 tx count
; common area in all banks    70..7F
; Interupt state Variables
SavedWR    ; EQU        0x70      ; IRQ saves W, STATUS reg, FSR and PCLATH
SavedSTAT  ; EQU        0x71      
SavedFSR   ; EQU        0x72
SavedPCLATH  ;          0x73      ; saved for high table nav stars
; scratchpad for calcs and displays
calc0      ; 0x74
calc1      ; 0x75
calc2      ; 0x76
calc3      ; 0x77
calc4      ; 0x78
calc5      ; 0x79
calc6      ; 0x7A
calc7      ; 0x7B
calc8      ; 0x7C
calc9      ; 0x7D
calc10     ; 0x7E
calc11     ; 0x7F

     endc

     cblock   0xA0

; stopwatch vars

lap0       ; 0xA0     0.1, stopwatch counter (kHz)
lap1       ; 0xA1     low  100Hz counter
lap2       ; 0xA2     mid
lap3       ; 0xA3     high
rec0       ; 0xA4     0.1 kHz of lap recorded
rec1       ; 0xA5     low
rec2       ; 0xA6     mid 
rec3       ; 0xA7     high
recnum     ; 0xA8     record number to store in EEPROM
runflag    ; 0xA9     0=stop nonzero=run
recbuf     ; 0xAA     buffer for recording stopwatch counter
   
     endc

LAPSTART  EQU  0x25                   ; timer2: x5, ON, x4 = 0x25; [x5 OFF x4 = 0x21]
LAPSTOP   EQU  0x21

DMSfact0  EQU  decsign

            ORG     0x0000

Start
            clrf    Dmode             ; mode 0
            clrf    navstar           ; first navigation star
            clrf    u2                ; clear uSec counter high byte may be enough
            goto    Initialise

            ORG     0x0004
IRQ_vector
            goto    IRQ_Start

; Message Table here so that it is all on one page *********
; call messages with data offset in w, returns character index.

; messages in 7seg code
messages        movwf PCL
;                    0123456789012345    0123456789012345   16 bytes per line
msg_saveEE      dt  5,0E,10,15,0E,0E,0              ; HEX VALUES!!
                 ;  "Set EE",0     ;     Set EE  ???? save values in EEPROM
msg_setlat 	dt  5,0E,10,15,11,0A,10,0
                 ;  "Set Lat",0    ;     Set Lat ???? set latitude to current DEC counter
msg_setdec 	dt  5,0E,10,15,0D,0E,0C,0
                 ;  "Set Dec",0    ;     Set Dec ???? set DEC to latitude & centre Hour Angle
msg_synch       dt 12,0E,0C,0A,11,11,0
                 ;  "Recall",0     ;     Recall    xx synch encoder counters to Set point
msg_setpt       dt  5,0E,10,15,13,10,0
                 ;  "Set Pt",0     ;     Set PT    xx save RA, DECenc as a set point
msg_ramax  	dt 12,0A,15,10,16,13,0
                 ;  "RA top ",0    ;     RA top 22196
msg_decmax 	dt 0D,0E,0C,10,16,13,0
                 ;  "DECtop",0     ;     DECtop 10045
msg_racount	dt 12,0A,15,0E,14,0C,0
                 ;  "RA Enc",0     ;     RA Enc 00567
msg_decount	dt 0D,0E,0C,15,0E,14,0
                 ;  "DEC En",0     ;     DEC En 02670
msg_time   	dt 10,16,0D,0
                 ;  "Tod ",0       ;     Tod hh mm ss
msg_sid    	dt  5, 1,0D,0
                 ;  "Sid  ",0      ;     Sid hh hh ss
msg_ra     	dt 12,0A,15,0
                 ;  "RA   ",0      ;     RA  hh mm ss
msg_dec    	dt 0D,0E,0C,0
                 ;  "Dec  ",0      ;     Dec -27 32.1
msg_timehr   	dt 10,16,0D,15,19,12,0
                 ;  "Tod hr ",0    ;     Tod hr    xx
msg_timemm   	dt 10,16,0D,15,14,14,0
                 ;  "Tod nn ",0    ;     Tod nn    xx
msg_sidhr   	dt  5, 1,0D,15,19,12,0
                 ;  "Sid hr ",0    ;     Sid hr    xx
msg_sidmm   	dt  5, 1,0D,15,14,14,0
                 ;  "Sid nn ",0    ;     Sid nn    xx
msg_welcome	dt 10,0E,11,0E, 5,0C,16,13, 1,0C,15, 1,0
                 ;  "TelescoPIC 1",0   ;    welcome screen
msg_navstar  	dt  5,0E,10,0
                 ;  "Set ",0       ;     Set AchErnAr  Navigation star
msg_edit        dt  0E,0D, 1,10,0
                 ;  "Edit"               Edit xx   xx   Edit a byte
msg_lap         dt  11,0A,13,0
                 ;  "LAP"                LAP 01234.56   Lap/Stopwatch
msg_rec         dt  12,0E,0C,0     ;     rec 0  655.35      
; Segments for LED displays:
; This sequence determined by my PCB layout rather than nice logical thinking!

#define _A  0x01     ; top bar for segment A , etc .. see PCB for details
#define _B  0x04     ; top rhs
#define _C  0x10     ; bot rhs
#define _D  0x80     ; bot bar
#define _E  0x20     ; bot lhs
#define _F  0x02     ; top lhs
#define _G  0x08     ; mid bar
#define _DP 0x40     ; dot rhs

led_segs ; call with character index in W: retrieves segment data for character
          addwf  PCL, f     ; add W to low part of program counter (computed jump)
          retlw _A+_B+_C+_D+_E+_F    ; ABCDEF. = '0'      ( # 0  )
          retlw    _B+_C             ; .BC.... = '1'      ( # 1  )
          retlw _A+_B   +_D+_E   +_G ; AB.DE.G = '2'      ( # 2  )
          retlw _A+_B+_C+_D      +_G ; ABCD..G = '3'      ( # 3  )
          retlw    _B+_C      +_F+_G ; .BC..FG = '4'      ( # 4  )
          retlw _A   +_C+_D   +_F+_G ; A.CD.FG = '5' 'S'  ( # 5  )
          retlw _A   +_C+_D+_E+_F+_G ; A.CDEFG = '6'      ( # 6  )
          retlw _A+_B+_C             ; ABC.... = '7'      ( # 7  )
          retlw _A+_B+_C+_D+_E+_F+_G ; ABCDEFG = '8'      ( # 8  )
          retlw _A+_B+_C+_D   +_F+_G ; ABCD.FG = '9' 'g'  ( # 9  )
          retlw _A+_B+_C   +_E+_F+_G ; ABC.EFG = 'A'    a ( # 10 )
          retlw       _C+_D+_E+_F+_G ; ..CDEFG = 'b'      ( # 11 )
          retlw          _D+_E   +_G ; ...DE.G = 'c'      ( # 12 )
          retlw    _B+_C+_D+_E   +_G ; .BCDE.G = 'd'      ( # 13 )
          retlw _A      +_D+_E+_F+_G ; A..DEFG = 'E'      ( # 14 )
          retlw _A         +_E+_F+_G ; A...EFG = 'F'      ( # 15 )
          retlw          _D+_E+_F+_G ; ...DEFG = 't'      ( # 16 )
          retlw          _D+_E+_F    ; ...DEF. = 'L'      ( # 17 )
          retlw             _E+   _G ; ....E.G = 'r'      ( # 18 )
          retlw _A+_B      +_E+_F+_G ; AB..EFG = 'P'      ( # 19 )
          retlw       _C   +_E   +_G ; ..C.E.G = 'n'      ( # 20 )
          retlw         0x00         ; ....... = ' '      ( # 21 )
          retlw       _C+_D+_E   +_G ; ..CDE.G = 'o'      ( # 22 )
          retlw                  _G  ; ......G = '-'      ( # 23 )
          retlw                  _DP ; .......x= '.'      ( # 24 )
          retlw       _C+   _E+_F+_G ; ..C.EFG = 'h'      ( # 25 )
          retlw _A+_B+_C+      _F+_G ; ABCDEFG = 'q'      ( # 26 )
          retlw       _C+_D+_E       ; ABCDEFG = 'u'      ( # 27 )
          retlw    _B+_C+_D+   _F+_G ; ABCDEFG = 'y'      ( # 28 )
          retlw       _C             ; ..C.... = 'i'      ( # 29 ) lower case i
; this next lot is for numbers with decimal points as in "0." (add 30 to the number)
          retlw _A+_B+_C+_D+_E+_F   +_DP; ABCDEF.d = '0.' ( # 30 )   andlw _DP   ;0x40
          retlw    _B+_C            +_DP; .BC....d = '1.' ( # 31 )
          retlw _A+_B   +_D+_E   +_G+_DP; AB.DE.Gd = '2.' ( # 32 )
          retlw _A+_B+_C+_D      +_G+_DP; ABCD..Gd = '3.' ( # 33 )
          retlw    _B+_C      +_F+_G+_DP; .BC..FGd = '4.' ( # 34 )
          retlw _A   +_C+_D   +_F+_G+_DP; A.CD.FGd = '5.' ( # 35 )
          retlw _A   +_C+_D+_E+_F+_G+_DP; A.CDEFGd = '6.' ( # 36 )
          retlw _A+_B+_C            +_DP; ABC....d = '7.' ( # 37 )
          retlw _A+_B+_C+_D+_E+_F+_G+_DP; ABCDEFGd = '8.' ( # 38 )
          retlw _A+_B+_C+_D   +_F+_G+_DP; ABCD.FGd = '9.' ( # 39 )
          ; Add more display codes here if needed !
; ASCII table for printable characters for above codes
ascii_tab  addwf   PCL, f
          dt "0123456789AbcdEFtLrPn o-.hquyi"

; LED address table: This scheme is to get 12 display digits.
; address is shifted out serially and decoded by 74HC138 + 4 direct lines
; This sequence determined by my PCB layout rather than nice logical thinking!
; bit 7,6,5 address digits 1-8, bit 4 is !enable for '138, bit3=digit9, bit2=digit10
; bit 1=digit11, bit0=digit12.
; Could be extended to 16 digits with two '138 decoders...
; Unused digits could have 8 leds atached to the segment drivers...

led_addr   addwf   PCL, f               ; 76543210         76543210
           retlw   0x0F                 ; 00001111  '138 = 11111110
           retlw   0x8F ;1F             ; 10001111         11111101
           retlw   0x4F ;2F             ; 01001111         11111011
           retlw   0xCF ;3F             ; 11001111         11110111
           retlw   0x2F ;4F             ; 00101111         11101111
           retlw   0xAF ;5F             ; 10101111         11011111
           retlw   0x6F                 ; 01101111         10111111
           retlw   0xEF ;7F             ; 11101111         01111111
           retlw   0x17                 ; 00010111  '138 disabled from here
           retlw   0x1B                 ; 00011011
           retlw   0x1D                 ; 00011101
           retlw   0x1E                 ; 00011110
                                        ; 
;--------------------------------------------------------------------------
; EXECUTION jump table for display mode 0-15.
;   depending on Dmode (Display Mode) Make sure Dmode doesn't exceed table.
;--------------------------------------------------------------------------
Dmode_job                               ; Execute the function belonging to Dmode
          ; call with Dmode in W register
          addwf  PCL, f                 ; add W to low part of program counter (computed jump)
          goto   display_ra_dec         ; 0  display RA/DEC        was time (mode0)
          goto   mode1                  ; 1  display RA  
          goto   mode2                  ; 2  display DEC
          goto   mode3                  ; 3  set point: copy decoders to eeprom
          goto   mode4                  ; 4  recall point: restore encoders from eeprom
          goto   mode5                  ; 5  set navigation star
          goto   mode6                  ; 6  Set declination to latitude
          goto   mode7                  ; 7  Set latitude as current declination
          goto   mode8                  ; 8  adjust RA encoder value
          goto   mode9                  ; 9  adjust DEC encoder value
          goto   mode10                 ; 10 adjust RA max encoder value
          goto   mode11                 ; 11 adjust DEC max encoder value
          goto   mode12                 ; 12 adjust Time - minutes
          goto   mode13                 ; 13 adjust Time - hours
          goto   mode14                 ; 14 adjust SidT - minutes
          goto   mode15                 ; 15 adjust SidT - hours
          goto   mode16                 ; 16 store values in EEPROM
          goto   mode17                 ; 17 Alternate encoder counts
          goto   mode18                 ; 18 Alternate time/sid
          goto   mode19                 ; 19 edit a variable
          goto   mode20                 ; 20 Lap/stopwatch function
          ; Add more jumps here if needed !
          goto   endmode                ; last entry is a switch to mode0 (time)
; 3 bytes left for table!

; Initialising code ****************************************

Initialise
            clrf    keyflag             ; no keys pressed yet
            clrf    ERRcount            ; skipped counts
            clrf    u1                  ; clear uSec counter mid byte
            CLRWDT                      ; clear watchdog - Actually, WDT is disabled
            CLRF    PORTA               ; clear A outputs
            CLRF    PORTB               ; clear B outputs
            movlw   0x07                ; turn comparators off
            movwf   CMCON               ; so RA pins can be used for I/O
            bcf     status,   RP1
            BSF     STATUS    , RP0     ; bank 1
            MOVLW   portb_DDR           ; crazy segmented intel-type code
            MOVWF   TRISB               ; portb  3 outputs  5 inputs
            MOVLW   porta_DDR           ; was 00
            MOVWF   TRISA               ; porta  1 output   4 inputs
            MOVLW   0x87                ; options: RB pullups off
            MOVWF   OPTION_REG          ; prescaler to timer, 1:256
            MOVLW   0x03                ; 03 Timer2: bit 1, + Timer 1: bit 0
            MOVWF   PIE1                ; overflow interupt enabled
            movlw   0xF9                ; Timer2 PR2 reg = 249 (gives 250 count)
            movwf   PR2
            BCF     STATUS    , RP0     ; bank 0
            clrf    st2                 ; clear sidtime counter
            clrf    dignum              ; display digit 0
            clrf    decsign             ; clear dec sign bit and stopwatch running flag bit
            movlw   0x01
            movwf   T1CON               ; turn on timer 1
            movlw   0xE0                ; -8192 counts = 1.6384 mSec
            movwf   TMR1H               ; set high byte to 224 for next rollover
            movlw   LAPSTOP             ; timer2: x5, ON, x4 = 0x25; [x5 OFF x4 = 0x21]
            movwf   T2CON               ; setup timer2 OFF
            movlw msg_welcome
            call char_strout            ; put welcome message directly in buffer
            CALL    read_EEPROM         ; copy eeprom contents to variables
            ; encoders set to last saved EEPROM values
            ;MOVLW   0x06                ; read initial encoder states
            ;ANDWF   PORTB     , W       ; could possibly delete this if we need the bytes
            ;MOVWF   oldDECen
            ;MOVLW   0x18
            ;ANDWF   PORTB     , W       ; we need the bytes...
            ;MOVWF   oldRAenc
set_rs232   CLRF    TMR0                ; clear timer0
            CLRF    COMMSFLAG           ; clear rs232flag
            BSF     PORTA     , 0       ; RA0 high (transmit = high) - idle state
            MOVLW   0x00
            MOVWF   PIR1                ; clear all IRQ flags
            MOVLW   0xF0                ; (was 0xB0) now GIE,PEIE,T0IE,INTE
            MOVWF   INTCON              ; enable GIE, per, Timer0 and external RB0 interupt
            call    init_LAP            ; clear stopwatch vars
; wait for MODE PB1 key press, then wait for it to be released
wait_key    call    Bit_Delay           ; 
            btfss   PORTA, PB_MODE      ; look at the nice welcome message
                                        ; wait for non-zero
            GOTO    wait_key
wait_key2   call    Bit_Delay           ; allow for bounce P1
            btfsc   PORTA, PB_MODE      ;
                                         ; wait for key release
            GOTO    wait_key2
Comms_ERR   MOVLW   0x45                ; 'E' could use this for errors in Main_Loop
Comms_reply MOVWF   PC_con              ; assume no PC
            CALL    TrX_byte            ; Are we all alone? Test to see if laptop present
                                        ; fall through to main loop

; Main Program loop ****************************************

; Repeat after me...
;   blink the seconds led
;   update RA & DEC encoders
;   check for rs232 comms
;   convert encoders to H M S, D M S
;   set up display buffers for LEDs
;   check for a keypress to change display mode or perform a task
; and repeat

Main_Loop

; check led: on for 1 sec, off for 1 sec.
            btfss   COMMSFLAG, 7        ; blink bit
            goto    main_setb           ; set it

            bcf     PORTA,  ledsec      ;clear it
            goto    main_blink

main_setb   bsf     PORTA,  ledsec
main_blink
; process encoders
            call    update_encoders

; check for rs232 incoming  (RB0 interrupt flags the start of a start bit)
            btfsc   COMMSFLAG, RXIP       ; bit 1
; if so, get it and process the message & send reply (check encoders  while doing so)
            goto    process_rs232
; update led disply message
            ; display RA,DEC can take many microSec so check for rs232 and
            ; update encoders between calls
            call    conv_RAcount        ; Set HAhms values from hour angle encoders

            call    update_encoders
            btfsc   COMMSFLAG, RXIP
            goto    process_rs232

            call    conv_DECcount       ; Set DECdms from encoders

            call    update_encoders
            btfsc   COMMSFLAG, RXIP
            goto    process_rs232
            
; process encoders - may need to inhibit interupt while accessing sid time

            call    calc_RAhms          ; find RAhms from HA + SID

; update display and check key press

            call    mode_switch         ; compose the right message in the calc scratchpad
            call    transfer_display    ; copy to display from calc... to buff...

; check for keypress PB1 (change mode)

            call    check_PB1           ; check if MODE pressed, (change mode)
            andlw 0xff                  ; set zero flag?
            btfsc   STATUS, Z           ; w=0 means not pressed (ignore)
            goto    Main_Loop           ; w=0, bailout
            ; change mode required, so clear keyflags and bump Dmode
            clrf    keyflag
            incf    Dmode, f            ; do the next display mode
            ; note: last valid mode just resets it to zero
            goto    Main_Loop


check_PB1   ; check for key press and release, return 1 if pressed AND released.
check_MODE  btfss   PORTA, PB_MODE
            goto    pbutm_clear
            bsf     keyflag, 7
            retlw   0

pbutm_clear btfss   keyflag, 7         ; was it 1->0?
            retlw   0
            bcf     keyflag, 7
            retlw   1


check_PB2   call    Delay_4m           ; 4 microsec debounce
check_PLUS  btfss   PORTA, PB_PLUS
            goto    pbut2_clear
            bsf     keyflag, 6
            retlw   0

pbut2_clear btfss   keyflag, 6         ; was it 1->0?
            retlw   0
            bcf     keyflag, 6
            retlw   1

check_PB3   call    Delay_4m           ; 4 microsec debounce
check_MINUS btfss   PORTA, PB_MINUS
            goto    pbut3_clear
            bsf     keyflag, 5
            retlw   0

pbut3_clear btfss   keyflag, 5         ; was it 1->0?
            retlw   0
            bcf     keyflag, 5
            retlw   1

check_PB4   call    Delay_4m           ; 4 microsec debounce
check_OK    btfss   PORTA, PB_OK
            goto    pbut4_clear
            bsf     keyflag, 4
            retlw   0

pbut4_clear btfss   keyflag, 4         ; was it 1->0?
            retlw   0
            bcf     keyflag, 4
            retlw   1


; mode display switch ladder. Make last mode a switch to mode 0.
; Display modes are:
; 0  display time
; 1  display RA
; 2  display Dec
; 3  Set point:    copy RA DEC encoders to EEPROM
; 4  Recall point: copy RA DEC encoders from EEPROM
; 5  select navigation star?
; 6  set declination to latitude & centre RA
; 7  set latitude to current declination
; 8  adjust RA encoder value
; 9  adjust DEC encoder value
; 10 adjust RA max encoder value
; 11 adjust DEC max encoder value
; 12 adjust Time - hours
; 13 adjust Time - minutes, press OK to zero sec
; 14 adjust SidT - minutes
; 15 adjust SidT - hours
; 16 store values in EEPROM
; 17 alternate encoders
; 18 Alternate time/sid
; 19 Edit variables...
; 20 Stopwatch lap timer
; compare position with target? (display difference in ra, dec)
; etc...

mode_switch                             ; select display from mode table

            call    clear_buffer        ; clean calc3 to end
            movf    Dmode, w            ; use jump table to execute mode task 0-15
            goto    Dmode_job           ; each modeXX ends with a return

endmode     clrf    Dmode               ; FALL THROUGH to mode 0
                                        ; it won't matter if we do mode18 once...
; mode 0 is display time of day

            ; mode=0 display time and [optional] alternate with siderial time
mode18      btfsc   secs, 1            ; check blinky bit
mode0       goto    display_hms
            goto    display_sid
            
            ; mode=1,2 : display RA,DEC 
;mode1      ; btfsc   secs, 1 ;COMMSFLAG, 7        ; check blinky bit
           ; goto    mode1_ra            ; switch between ra, dec each second
mode2       call    display_dec         ; dec display, with switch back to ra
            goto    mode_pr             ; exit via '-' check
mode1
mode1_ra    call    display_ra          ; display ra, enable return to time of day
                                        ; via 'mode-previous' check
mode_pr     call    check_PB3           ; returns w=1 if pb3 pressed and released '-'
		andlw 0xff              ; set zero flag?
            btfss   STATUS, Z           ; w=0 means PB3 NOT pressed
            decf    Dmode, F            ; previous mode
            return                      

            ; Set point: save current position in EEprom
mode3       movlw   msg_setpt
            call    lcd_strout
con_ptnum  ; call    clear_buffer5
            movf    pointnum, W
            call    con_2digits
            movlw   pointnum
            call    edit_8              ; allow + and - keys to change setpoint
            movlw   0x07                ; clip 0-7
            andwf   pointnum, f
            call    check_PB4           ; check if OK pressed
		andlw 0xff              ; set zero flag?
            btfsc   STATUS, Z           ; non zero
            return
            call    save_Setpoint       ; copy RA DEC encoders to EEPROM
mode_radec  movlw   d'1'                ; set up return to RA/Dec display
            movwf   Dmode
            return

            ; Recall a saved set point
mode4       movlw   msg_synch      
            call    lcd_strout
           ; call    clear_buffer5       ; display pointnum
            movf    pointnum, W
            call    con_2digits
            movlw   pointnum
            call    edit_8              ; allow + and - keys to change setpoint
            movlw   0x07                ; clip 0-7
            andwf   pointnum, f
            call    check_PB4           ; check if OK pressed
		andlw 0xff              ; set zero flag?
            btfsc   STATUS, Z           ; non zero
            return
            call    read_Setpoint       ; copy RA, DEC encoder from EEPROM
            goto    mode_radec          ; set up return to RA/Dec display


mode5       movlw   msg_navstar         ; setup navigation star stored in ROM
            call    lcd_strout          ; "Set "
            movf    navstar, W
            call    get_nav_name        ; "Achernar"
;            call    transfer_display    ; copy from calc... to buff...
            movlw   navstar
            call    edit_8              ; allow + and - keys to change navstar
            movlw   0x0F                ; clip 0-15
            andwf   navstar, f
            call    check_PB4           ; check OKAY button
		andlw 0xff              ; set zero flag?
            btfsc   STATUS, Z           ; non zero
            return
            call    get_navstar         ; copy navigation star to RA/DEC
            goto    mode_radec          ; set up return to RA/Dec display
            

            
mode6       movlw   msg_setdec          ; set declination to latitude & centre RA
            call    lcd_strout
            ;call    clear_buffer5       ; clear rubbish
            call    check_PB4           ; check OKAY key
		andlw 0xff              ; set zero flag?
            btfsc   STATUS, Z           ; non zero
            return
            clrf    Dmode               ; (revert to default time dislay after setting RA/Dec)
            call    set_RAmid           ; middle hourangle encoder
            goto    set_DECLAT          ; set dec encoder to stored lattitude
            
mode7       movlw   msg_setlat          ; set latitude to current declination
            call    lcd_strout
            call    clear_buffer5       ; clear rubbish
            call    check_PB4
		andlw 0xff              ; set zero flag?
            btfsc   STATUS, Z           ; non zero
            return
            call    set_LATDEC          ; set stored lattitude from DEC encoder
            goto    mode_radec          ; set up return to RA/Dec display

            ; adjust RA encoder value
            ; BUG: why does this not change the numbers?
            ; is the enoder read stopping the adjustment?
mode8       call    display_RAenc     
            movlw   RAenc
            goto    edit16

            ; adjust DEC encoder value
mode9       call    display_DECenc
            movlw   DECenc     

; edit a 16 bit number if PLUS or MINUS key pressed.
; PLUS key increments, MINUS key decrements

edit16      movwf   FSR                ; use indirect addressing to edit 16bit number
            call    check_PB2          ; returns w=1 if pb2 pressed and released '+'
            andlw   0xff               ; set zero flag?
            btfss   STATUS, Z          ; Zset (w=0) means nobody pressed PB2 key
            goto    inc_16
            call    check_PB3          ; returns w=1 if pb3 pressed and released '-'
            andlw   0xff               ; set zero flag?
            btfsc   STATUS, Z          ; w!=0 means PB3 pressed
            return
dec_16      decfsz  INDF, F            ; 16 bit decrement, nearly
            return                     ; should do a subtraction to avoid the bugs
            incf    FSR, f
            decf    INDF, f
            return

inc_16      incfsz  INDF, F            ; increment 16 bit
            return
            incf    FSR, f
            incf    INDF, f
            return


            ; adjust RA max encoder value
mode10      call    display_RAmax     
            movlw   RAmax
            goto    edit16


            ; adjust DEC max encoder value
mode11      call    display_DECmax     
            movlw   DECmax
            goto    edit16


mode13                              
            movlw   msg_timemm
            call    lcd_strout
            call    con_timeMin 
            movlw   mins
            call    edit_8
            call    check_PB4           ; check OKAY button to zero seconds
            andlw   0xff                ; set zero flag?
            btfsc   STATUS, Z           ; non zero
            return
            clrf    secs                ; zero the seconds on pressing OK button
            clrf    u2                  ; including the microsec counter
            clrf    u1
            goto    endmode             ; return to time display

; edit an 8 bit number if PLUS or MINUS key pressed.
; PLUS key increments, MINUS key decrements

edit_8      movwf   FSR                ; use indirect addressing to edit 8 bit number
            call    check_PB2          ; returns w=1 if pb2 pressed and released '+'
            andlw   0xff               ; set zero flag?
            btfss   STATUS, Z          ; Zset (w=0) means nobody pressed PB2 key
            goto    inc_8 
            call    check_PB3          ; returns w=1 if pb3 pressed and released '-'
            andlw   0xff               ; set zero flag?
            btfss   STATUS, Z          ; w=0 means PB3 NOT pressed
dec_8       decf    INDF, F            ; 8 bit decrement, nearly
            return                     ; should do a subtraction to avoid the bugs

inc_8       incf    INDF, F            ; increment 8 bit
            return


mode12      ; edit time of day - hours                        
            movlw   msg_timehr
            call    lcd_strout
            call    con_timeHour
            movlw   hours
            goto    edit_8

mode15      ; edit sid minutes                        
            movlw   msg_sidmm
            call    lcd_strout
            call    con_sidMin  
            movlw   sidmin
            goto    edit_8

mode14      ; edit  display sid hours
            movlw   msg_sidhr
            call    lcd_strout
            call    con_sidHour
            movlw   sidhr
            goto    edit_8

mode16      ; save current variables in EEPROM!
         ;   call    clear_buffer        ; clean display
            movlw   msg_saveEE          ; store values in EEPROM
            call    lcd_strout
            call    check_PB4           ; check OKAY button
            btfsc   STATUS, Z           ; non zero
            return
            call    save_EEPROM         ; copy 17 values to EEPROM
            clrf    Dmode               ; revert to mode0 when finished
            return

            ; mode=17 display RA,DEC counters 
mode17       btfsc   secs, 1 ;COMMSFLAG, 7        ; check blinky bit
            goto    mode2_ra            ; RA encoder check
            call    display_DECenc
            goto    mode_pr             ; exit via '-' check

mode2_ra    call    display_RAenc
            goto    mode_pr             ; exit via '-' check
            
mode19      ; edit any variable 0x20...0x7f                        
            movlw   msg_edit
            call    lcd_strout
            movf    editloc, w
            call    con_hex
            movwf   calc10  
            movlw   editloc
            call    edit_8              ; allow + and - keys to change variable location
            movf    editloc, W
            btfsc   editloc, 7
            movlw   0x20
            movwf   editloc
            call    check_PB4           ; check if OK pressed
		andlw 0xff              ; set zero flag?
            btfsc   STATUS, Z           ; Z w=0 means not pressed
            return
            ; editloc contains the address of the variable to edit
            ; put the current value of the 'target' to editval and adjust that.
            ; if the user presses return, copy the new value back to the target variable.
            ; this part will not exit until return is pressed again, so we need to
            ; keep updating the display!
            clrf    keyflag
            movf    editloc, w          ; copy target variable to editval
            movwf   FSR
            movf    INDF, w
            movwf   editval
edit_chk    call    clear_buffer        ; update display each time
            movlw   msg_edit
            call    lcd_strout
            movf    editloc, w
            call    con_hex
            movwf   calc6
            movf    calc11, w
            movwf   calc7   
            movf    editval, w
            call    con_hex
            movwf   calc10  
            call    transfer_display    ; copy to display...
            call    Start_Delay         ; debounce time?

            movlw   editval            ; allow +/- editing
            call    edit_8
            call    check_PB1           ; check if MODE pressed, (cancel edit)
            andlw 0xff                  ; set zero flag?
            btfss   STATUS, Z           ; w=0 means not pressed (ignore)
            goto    edit_exit           ; bailout
            call    check_PB4           ; check if OK pressed
            andlw 0xff                  ; set zero flag?
            btfsc   STATUS, Z           ; non zero means OK was pressed
            goto    edit_chk            ; repeat until OK pressed
            ; if OK pressed, copy value to target
            movf    editloc, w
            movwf   FSR
            movf    editval, w
            movwf   INDF
edit_exit
           ; clrf    Dmode               ; exit back to the ranch
            return

mode20                                  ; lap counter/stopwatch
            call    display_LAP         ; set up display         
            ; check start button  '+'
            call    check_PB2           ; check '+' button
		andlw 0xff              ; set zero flag?
            btfsc   STATUS, Z           ; non zero means start pressed
            goto    mode20_1
            goto    start_LAP           ; start/reset counter
mode20_1           ; check action key '-'
            call    check_PB3           ; check '-' button
		andlw 0xff              ; set zero flag?
            btfsc   STATUS, Z           ; non zero
            goto    mode20_2
            goto    log_LAP             ; log the record
mode20_2            ; check 'OK' button to stop & record in EEPROM
            call    check_PB4           ; check OKAY button
		andlw 0xff              ; set zero flag?
            btfsc   STATUS, Z           ; non zero
            return
            goto    stop_LAP            ; copy navigation star to RA/DEC


transfer_display                        ; copy calc... to buff...
            movf    calc0, w
            movwf   buff0
            movf    calc1, w
            movwf   buff1
            movf    calc2, w
            movwf   buff2
            movf    calc3, w
            movwf   buff3
            movf    calc4, w
            movwf   buff4
            movf    calc5, w
            movwf   buff5
            movf    calc6, w
            movwf   buff6
            movf    calc7, w
            movwf   buff7
            movf    calc8, w
            movwf   buff8
            movf    calc9, w
            movwf   buff9
            movf    calc10, w
            movwf   buff10
            movf    calc11, w
            movwf   buff11
            return

; Seven Segment LED code.  ********************************************
; 12 characters, 3 wire serial interface

; To copy a character string from program memory, you set up a register to point to the
; first character, (ptr), and repeatedly call "messages", incrementing the pointer each time.
; We have reached the end of the string when a zero is returned.
; For 7 seg displays, this cannot be the index for "0" since this is zero.
; The routine is entered with the offset of the first character in 'W':
; The characters are transfered to a RAM buffer for IRQ to display.
; TIMER1 IRQ sends the characters to the led shift registers every 2mS.
 
; send a string to the LED buffers

char_strout
                ; send string to the visible LED buffer.
                ; We send the welcome message to the display buffer directly.
                movwf ptr
                movlw buff0
                goto  str_xxx
lcd_strout
                ; send string to the non visible calc buffer. This removes the flickering
                ; we get when we send to the display buffer directly. Remember to copy
                ; from calc0... to buff0... after completion.
led_strout	movwf ptr
                movlw    calc0
str_xxx         movwf    FSR
str_00		movf ptr, W
		call messages
		andlw 0xff              ; set zero flag?
		bz str_01
		movwf    INDF           ; output char in W to buffer
                incf     FSR, f         ; next destination
		incf ptr, F             ; next char
		goto str_00
str_01		return

lcd_buf8        return                  ; dummy return. Removed lcd code



; Conversion and display routines:

; convert byte in W reg to two hex characters, in calc10 and W
; Enter with character in W
; Returns with lo nibble char in W, hi char stored in calc10 (hh mm *s)

con_hex
                movwf   lcd_buf1        ; temp save
                andlw   0x0F            ; lo nibble
                movwf   calc11
                swapf   lcd_buf1, w
                andlw   0x0F            ; high nibble
                return

;  ascii version not needed?
;con_hex8	movwf	lcd_buf1        ; save the value
;		swapf	lcd_buf1, W
;		call	con_hxout       ; do upper nibble
;                movwf   calc10
;		movf	lcd_buf1, W     ; and lower one
;con_hxout	andlw	0x0f
;		addlw	0xf6
;		btfsc	STATUS, C
;		addlw	0x07
;		addlw	0x3a
;		return                  ; goto lcd_chrout

; convert a byte to a 2 or 3 decimal txt string in calcer5, calc10 and W

con_decout2     movwf   lcd_buf2        ; 2 char string "21" or "00"
;                goto    con_dec22

;con_decout	movwf	lcd_buf2        ; 3 char string "241"  "007"
;		movlw	'0'
;		movwf	lcd_buf1

;con_dec100	movlw	d'100'
;		subwf	lcd_buf2, W
;		bnc	con_dec10

;		movwf	lcd_buf2
;		incf	lcd_buf1, F
;		goto	con_dec100

;con_dec10	movf	lcd_buf1, W
;		movwf   calc9         ;call	lcd_chrout

;  should use seg code index to make display code simpler for 7 seg displays

con_dec22	movlw	d'0'            ; 0 is seg code index for "0"
		movwf	lcd_buf1

con_dec11	movlw	d'10'
		subwf	lcd_buf2, W
		bnc	con_dec1

		movwf	lcd_buf2
		incf	lcd_buf1, F
		goto	con_dec11

con_dec1	movf	lcd_buf1, W
		movwf   calc10         ;call	lcd_chrout

		movlw	d'0'
		addwf	lcd_buf2, W
		return                  ; exit with char 0-9 in W for 7 seg led
		
; set up: mov low to lcd_dly1 and Hi to W and call con_dec16

; USAGE:     MOVF    DECmax  , W
;            MOVWF   lcd_dly1
;            MOVF    DECmaxHi, W
;            CALL    con_dec16           ; conv->decimal 00000
;            GOTO    calc4 display routine

con_dec16                        ; convert 16 bits to 22000 at calcer1...5
            MOVWF   lcd_dly2
            call    clear_buffer5
            MOVLW   0x27                ; $2710 = 10,000  26 words
            MOVWF   lcd_buf2
            MOVLW   0x10
            MOVWF   lcd_buf1
            CALL    digit16
            MOVWF   calc7 ;1
            MOVLW   0x03                ; $03E8 = 1,000
            MOVWF   lcd_buf2
            MOVLW   0xE8
            MOVWF   lcd_buf1
            CALL    digit16
            MOVWF   calc8 ;2
con_dec08   CLRF    lcd_buf2                ; $0064 = 100
            MOVLW   0x64
            MOVWF   lcd_buf1
            CALL    digit16
            MOVWF   calc9 ;3
            CLRF    lcd_buf2             ; $000A = 10
            MOVLW   0x0A
            MOVWF   lcd_buf1
            CALL    digit16
            MOVWF   calc10 ;4
            MOVF    lcd_dly1  , W
          ;  ADDLW   0x30                ; 7seg index, not ascii!
            MOVWF   calc11 ;5
            RETURN

digit16     MOVLW   0x00                ; 0 in 7seg code is '0' ascii code
            MOVWF   XREG
            goto    digitloop2
digitloop   INCF    XREG  , f
digitloop2  MOVF    lcd_buf1  , W       ; copy to work area
            SUBWF   lcd_dly1  , f       ; subtract low bytes
                MOVF    lcd_buf2  , W       ; look at high byte
            BTFSS   STATUS    , C       ; carry set = ok
            ADDLW   0x01                ;  cc, borrow = add in the borrow
            SUBWF   lcd_dly2  , f       ; subtract high bytes
            BTFSC   STATUS    , C       ; check if underflowed
            GOTO    digitloop           ;cs, ok to continue
            MOVF    lcd_buf1  , W       ; underflowed so add factor back in
            ADDWF   lcd_dly1  , f       ; low byte first
            MOVF    lcd_buf2  , W
            BTFSC   STATUS    , C       ; check overflow
            ADDLW   0x01                ; and fix high byte
            ADDWF   lcd_dly2  , f
            MOVF    XREG, W             ; exit with count in w
            RETURN

clear_buffer    movlw   d'21'           ; was 0x20    space
                movwf   calc3
                movwf   calc4
                movwf   calc5
                movwf   calc6
clear_buffer5   movlw   d'21'           ; was 0x20    space
                movwf   calc7
clear_buffer4   movlw   d'21'           ; was 0x20    space
                movwf   calc8
                movwf   calc9
                movwf   calc10
                movwf   calc11
                return


con_timeHour    call    clear_buffer5
                movf    hours, W
                goto    con_2digits

con_timeMin     call    clear_buffer5
                movf    mins, W
                goto    con_2digits

display_hms     movlw   msg_time
                call    lcd_strout

con_timeHMS     call    clear_buffer
                movf    hours, W
                call    con_decout2
                movwf   calc5
                movf    calc10, w
                movwf   calc4
                movf    mins, W
                call    con_decout2
                movwf   calc8
                movf    calc10, w
                movwf   calc7
                movf    secs, W
con_2digits     call    con_decout2
                movwf   calc11
                return

con_sidHour     call    clear_buffer5
                movf    sidhr, W
                goto    con_2digits

con_sidMin      call    clear_buffer5
                movf    sidmin, W
                goto    con_2digits

display_sid     movlw   msg_sid
                call    lcd_strout

con_sidtimeHMS  call    clear_buffer
                movf    sidhr, W
                call    con_decout2
                movwf   calc5
                movf    calc10, w
                movwf   calc4
                movf    sidmin, W
                call    con_decout2
                movwf   calc8
                movf    calc10, w
                movwf   calc7
                movf    sidsec, W
                call    con_decout2
                movwf   calc11
                return

display_ra      movlw   msg_ra
                call    lcd_strout

con_ra          call    clear_buffer
                movf    RAhh, W
                call    con_decout2
                movwf   calc5
                movf    calc10, w
                movwf   calc4
                movf    RAmm, W
                call    con_decout2
                movwf   calc8
                movf    calc10, w
                movwf   calc7
                movf    RAss, W
                call    con_decout2
                movwf   calc11
                return

display_dec     movlw   msg_dec
                call    lcd_strout

con_dec         call    clear_buffer
                movlw   d'21'           ; 7 seg index for 'space' - for "+"
                btfsc   decsign, 7      ; 0=north, 1=south. was DECdd, 7
                movlw   d'23'           ; 7 seg index to '-'
                movwf   calc4
                movf    DECdd, w
                andlw   0x7F            ; insurance, can be deleted
                call    con_decout2
                movwf   calc6
                movf    calc10, w
                movwf   calc5
                movf    DECmm, W
                call    con_decout2
                movwf   calc9
                movf    calc10, w
                movwf   calc8
                movlw   d'24'           ; 7 seg index to decimal point '.'
                movwf   calc10
                movf    DECss, W
                andlw   0x0f           ; insurance ????????????
                movwf   calc11
                return

display_ra_dec
               ; trim this to 30 instructions...
               ; call con-dec
               ; move dec -ddmm.s to tail end
               ; put future calc10 in a safe place
               ; then do ra hh mm ss
               ; and restore calc10
                call    con_dec
                movf    calc9, w
                addlw   d'30'
                movwf   editval        ; temp safe place
                movf    calc8, w
                movwf   calc9
                movf    calc6, w
               ; addlw   d'30'
                movwf   calc8
                movf    calc5, w
                movwf   calc7
                movf    calc4, w
                movwf   calc6

                movf    RAhh, W
                call    con_decout2
               ; addlw   d'30'
                movwf   calc1
                movf    calc10, w
                movwf   calc0
                movf    RAmm, W
                call    con_decout2
               ; addlw   d'30'
                movwf   calc3
                movf    calc10, w
                movwf   calc4
                movf    RAss, W
                call    con_decout2
                movwf   calc5
                movf    calc10, w
                movwf   calc4
                movf    editval, w
                movwf   calc10
                return

;                movlw   d'21'           ; 7 seg index for 'space' - for "+"
;                btfsc   decsign, 7      ; 0=north, 1=south. was DECdd, 7
;                movlw   d'23'           ; 7 seg index to '-'
;                movwf   calc6
;                movf    DECdd, w
;                andlw   0x7F            ; insurance, can be deleted
;                call    con_decout2
;                addlw   d'30'
;                movwf   calc8
;                movf    calc10, w
;                movwf   calc7
;                movf    DECmm, W
;                call    con_decout2
;                addlw   d'30'
;                movwf   calc11
;                movf    calc10, w
;                movwf   calc9
;              ;  movlw   d'24'           ; 7 seg index to decimal point '.'
;                movf    calc11, w
;                movwf   calc10
;                movf    DECss, W
;                andlw   0x0f           ; insurance ????????????
;                movwf   calc11
;                return

;con_counters    call    clear_buffer
;                movf    DECcount
;                call    con_hex8
;                movwf   calc5
;                movf    calc10, w
;                movwf   calc4
;                movf    DECcountHi
;                call    con_hex8
;                movwf   calc7
;                movf    calc10, w
;                movwf   calc6
;                movf    RAcount
;                call    con_hex8
;                movwf   calc9
;                movf    calc10, w
;                movwf   calc8
;                movf    RAcountHi
;                call    con_hex8
;                movwf   calc11
;                return



display_RAenc   movlw   msg_racount
                call    lcd_strout
                movf    RAcount, w
                movwf   lcd_dly1
                movf    RAcountHi, w
                goto    con_dec16

display_DECenc  movlw   msg_decount
                call    lcd_strout
                movf    DECcount, w
                movwf   lcd_dly1
                movf    DECcountHi, w
                goto    con_dec16

display_RAmax   movlw   msg_ramax
                call    lcd_strout
                movf    RAmax, w
                movwf   lcd_dly1
                movf    RAmaxHi, w
                goto    con_dec16

display_DECmax  movlw   msg_decmax
                call    lcd_strout
                movf    DECmax, w
                movwf   lcd_dly1
                movf    DECmaxHi, w
                goto    con_dec16

; STOPWATCH & LAP TIMER CODE

; In lap timer mode, a counter in KHz is incremented each interupt. Every tenth count
; is used in a 100Hz counter up to 3 bytes long.
; When a button is pressed, this counter can be copied to RAM and displayed on the LEDs.
; as   "LAP 0 234.56"
; Button '+' could be "start/reset counter"
;        '-'          "record lap time"
;        'OK'         "stop counter and save in EEPROM"
; In order to fit this in, I've trimmed it back to display a max count of 65535 (655.35 secs)
; but with three bytes of storage per record.


display_LAP ; change display from "LAP..." when off, to "rec..." when running

           ; bsf     status, RP0         ; bank 1   
            btfss   decsign, 6
            goto    disp20_0
           ; bcf     status, RP0         ; bank 0
            movlw   msg_rec
            goto    disp20_1
disp20_0   ; bcf     status, RP0
            movlw   msg_lap
disp20_1    call    lcd_strout
            ; works up to here
            BCF     INTCON    , GIE     ; clear GIE, prevent all interupt vectors
            bsf     status, RP0      ; bank 1
            movf    rec1, w
            bcf     status, RP0      ; bank 0
            movwf   lcd_dly1
            bsf     status, RP0      ; bank 1
            movf    rec2, w
            bcf     status, RP0      ; bank 0
            BSF     INTCON    , GIE     ; set GIE, restore all interupt vectors
            call    con_dec16        ; display as a 16 bit count
            BCF     INTCON    , GIE     ; clear GIE, prevent all interupt vectors
            bsf     status, RP0      ; bank 1
            movf    recnum, w
            bcf     status, RP0      ; bank 0
            BSF     INTCON    , GIE     ; set GIE, restore all interupt vectors
            movwf   calc4
            movlw   0x1E             ; add a decimal point by adding 30
            addwf   calc9, f         ;9 to the hundreds digit
            return

init_LAP    ; put lap vars in initial state
            BCF     INTCON    , GIE     ; clear GIE, prevent all interupt vectors
            movlw   0x21                ; timer2: x5, ON, x4 = 0x25; [x5 OFF x4 = 0x21]
            movwf   T2CON               ; switch off timer2
            bcf     decsign, 6          ; clear the "stopwatch running" flag
            bsf     status, RP0      ; bank 1
            clrf    lap0
            clrf    lap1
            clrf    lap2
            clrf    lap3
            clrf    rec1
            clrf    rec2
            clrf    rec3
            clrf    recnum              ; set the first record to 0
            bcf     status, RP0      ; bank 0
            BSF     INTCON    , GIE     ; restore all interupt vectors
            return


start_LAP   ; start stopwatch
            ; stop it first, clear count and start it up.
            call    init_LAP            ; stop timer, clear vars
            BCF     INTCON    , GIE     ; clear GIE, prevent all interupt vectors
           ; bsf     status, RP0      ; bank 1
            bsf     decsign, 6          ; set a flag to indicate that we are running
           ; bcf     status, RP0      ; bank 0
            BSF     INTCON    , GIE     ; restore all interupt vectors
            movlw   0x25                ; timer2: x5, ON, x4 = 0x25; [x5 OFF x4 = 0x21]
            movwf   T2CON               ; switch on timer2
            return

stop_LAP    ; stop stopwatch and save records to eeprom
            ; leave count as is until next start.
            movlw   0x21                ; timer2: x5, ON, x4 = 0x25; [x5 OFF x4 = 0x21]
            movwf   T2CON               ; switch off timer2
            call    log_LAP
            BCF     INTCON    , GIE     ; clear GIE, prevent all interupt vectors
           ; bsf     status, RP0      ; bank 1
            bcf     decsign, 6
           ; bcf     status, RP0      ; bank 0
            BSF     INTCON    , GIE     ; restore all interupt vectors
            ; copy to EEPROM
            goto    save_LAP
            ;return                      

log_LAP     ; log the stopwatch count to the rec0... bytes for display
            ; and copy to ram buffer in page 1
            btfss   decsign, 6        ; are we running?
            goto    start_LAP         ; no, so do start routine instead
            BCF     INTCON    , GIE     ; clear GIE, prevent all interupt vectors
            bsf     status, RP0      ; bank 1
           ; movf    lap0, w          ; don't need to copy the kHz part
           ; movwf   rec0             ; but the rest can be copied to display bytes
            movf    lap1, w           ; copy the count
            movwf   rec1
            movf    lap2, w
            movwf   rec2
            movf    lap3, w
            movwf   rec3
            ; copy to recording buffer
            movlw   recbuf
            addwf   recnum, w
            addwf   recnum, w
            addwf   recnum, w
            movwf   FSR
            movf    rec1, w
            movwf   INDF
            incf    FSR, f
            movf    rec2, w
            movwf   INDF
            incf    FSR, f
            movf    rec3, w
            movwf   INDF
            ; bump recording number
            incf    recnum, w
            andlw   0x0F
            movwf   recnum
            bcf     status, RP0      ; bank 0
            BSF     INTCON    , GIE     ; restore all interupt vectors
            return


; PC messages via rs232 ************************************

process_rs232                           ; goto'd from Mainloop
            CALL    Rcv_RS232           ; get byte in W, rxbuff2
            MOVWF   PC_con              ; safe place
            SUBLW   0x7A                ; 'z'
            BTFSC   STATUS    , Z
            GOTO    Rx_maxres           ; expect ABCD encoder resolution
            MOVF    PC_con   , W
            SUBLW   0x64                ; 'd'
            BTFSC   STATUS    , Z
            GOTO    Rx_radec            ; expect HMSDMS RA and DEC position
            MOVF    PC_con    , W
            SUBLW   0x79                ; 'y'
            BTFSC   STATUS    , Z
            GOTO    send_position       ; report encoder position ABCD
            MOVF    PC_con    , W
            SUBLW   0x70                ; 'p'
            BTFSC   STATUS    , Z
            GOTO    send_Err            ; report errors
            MOVF    PC_con    , W
            SUBLW   0x68                ; 'h'
            BTFSC   STATUS    , Z
            GOTO    send_maxres         ; report encoder resolution ABCD
                                        ; else unrecognised code
            GOTO    Comms_ERR           ; send 'E' via Main_Loop


; process messages via rs232

; receive:  followed by:  reply
;  'z'       ABCD         'r'    download encoder max AB-dec CD=ra, low first
;  'h'       ----         ABCD   upload encoder max limits to PC (as above)
;  'y'       ----         ABCD   upload encoder counts to PC, format as above
;  'p'       ----         A      upload error to PC (skipped counts)
;  'd'      HMSDMS        'd'    download RA and DEC for lcd display
;   -        ----         'E'    comms error - unrecognised first character

; The 'E' code is also sent at program startup.

; generic wait for next rs232 character.
; call this when expecting a sequence of characters
; exits with received char in W, rxbuf2 (and rxbuf, but next RB0 interupt will
; clear it)
; we need a time out or the project will wait forever without updating led.

wait_message
            movwf   FSR                 ; use indirect addressing to save char
            clrf    lcd_dly1            ; use as timeout counter (not saved if error)
            goto    wait_start

wait_read   CALL    update_encoders     ; read encoders (few uSec delay)
wait_start  incf    lcd_dly1, f         ; loop 256 times as a timeout
            btfsc   STATUS, Z           ; check FF->00 rollover
            retlw   0                   ; timeout when zero reached, return without updating
wait_next   BTFSS   COMMSFLAG , RXIP    ; check rs232 flag for incoming
            GOTO    wait_read           ; keep looking
            BCF     COMMSFLAG , RXIP    ; acknowledged and cleared
            CALL    Rcv_RS232           ; get updated value and clear RXIP flag
            movwf   INDF                ; save via indirect addressing
            return                      ; return with byte in W (could also be 0)

Rx_maxres                               ; receive encode MAX counts from PC
            MOVLW   DECmax              ; call wait_message with address of target
            CALL    wait_message
            MOVLW   DECmaxHi
            CALL    wait_message
            MOVLW   RAmax
            CALL    wait_message
            MOVLW   RAmaxHi
            CALL    wait_message        ; done?
            CALL    save_EEPROMmax
            MOVLW   0x72                ; 'r'
;            CALL    TrX_byte
            GOTO    Comms_reply         ; acknowlege via Main_Loop

Rx_radec                                ; receive RA and DEC from PC
            MOVLW   RAhh
            CALL    wait_message
            MOVLW   RAmm
            CALL    wait_message
            MOVLW   RAss
            CALL    wait_message
            MOVLW   DECdd
            CALL    wait_message
            MOVLW   DECmm
            CALL    wait_message
            MOVLW   DECss
            CALL    wait_message
            MOVLW   0x64                ; 'd'   acknowlege okay
;            CALL    TrX_byte
            GOTO    Comms_reply         ; acknowledge via Main_Loop

send_Err    MOVF    ERRcount  , W
            CLRF    ERRcount
;            CALL    TrX_byte            ; TX char in W then read encoders
            GOTO    Comms_reply         ; send err count via Main_Loop

;send_Err_code
;            MOVF    ERRcount  , W
;            ADDLW   0x30
;            MOVWF   trxbuff
;            CALL    TrX_byte
;            MOVLW   0x30
;            MOVWF   trxbuff
;            CALL    TrX_byte
;            MOVLW   0x31
;            MOVWF   trxbuff
;            CALL    TrX_byte
;            MOVLW   0x0D
;            MOVWF   trxbuff
;            CALL    TrX_byte
;            CLRF    ERRcount
;            GOTO    Main_Loop

send_position                           ; send (TX) encoders as binary codes
            MOVF    DECcount  , W
            CALL    TrX_byte            ; TX char in W then read encoders
            MOVF    DECcountHi, W
            CALL    TrX_byte            ; TX char in W then read encoders
            MOVF    RAcount   , W
            CALL    TrX_byte            ; TX char in W then read encoders
            MOVF    RAcountHi , W
;            CALL    TrX_byte            ; TX char in W then read encoders
            GOTO    Comms_reply         ; send via Main_Loop

send_maxres
            MOVF    DECmax    , W
            CALL    TrX_byte            ; TX char in W then read encoders
            MOVF    DECmaxHi  , W
            CALL    TrX_byte            ; TX char in W then read encoders
            MOVF    RAmax     , W
            CALL    TrX_byte            ; TX char in W then read encoders
            MOVF    RAmaxHi   , W
;            CALL    TrX_byte            ; TX char in W then read encoders
            GOTO    Comms_reply         ; send via Main_Loop

; rs232 transmitting code ************************************

; transmit character in W reg - at 9600 baud, should take 1mSec or so.
; use TrX_byte if you want to check the encoders after transmission.

TrX_byte    CALL    XMIT_RS232          ; send char in W
            GOTO    update_encoders     ; return via encoder routine

XMIT_RS232  MOVWF   trxbuff             ; move W to Xmit_Byte
            bsf     COMMSFLAG, TXIP     ; Tx comms in progress
            MOVLW   0x08                ; set 8 bits out
            MOVWF   bitcount
            BCF     PORTA,  1           ; RA1 low for start bit
            CALL    Bit_Delay
Ser_Loop    RRF     trxbuff , f         ; send one bit, rotate into carry flag
            BTFSS   STATUS    , C       ; skip if bit is 1
            BCF     PORTA, 1            ; bit = 0 so RA1 = 0
            BTFSC   STATUS    , C       ; forced skip ahead
            BSF     PORTA, 1            ; bit = 1 so RA1 = 1
            CALL    Bit_Delay
            DECFSZ  bitcount  , f       ;test if all done
            GOTO    Ser_Loop
            BSF     PORTA, 1            ; stop bit returns RA1 to '1'
            bcf     COMMSFLAG, TXIP     ; clear bit: Tx comms no longer in progress
            goto    Bit_Delay
            ;RETURN                      ; ready for next if required

; Receive byte from rs232, here because interupt RB0 fired and COMMSFLAG set bit 1
; RB0 interrupt disabled during receive, restore it when done, exit with W=char.

Rcv_RS232   CALL    Start_Delay         ;do half bit time delay
            MOVLW   0x08                ;set up to read 8 bits
            MOVWF   bitcount
            CLRF    rxbuff
Next_RcvBit CALL    Bit_Delay           ; should be middle of bit
            BTFSS   PORTB, 0            ; skip if RB0=1
            BCF     STATUS    , C       ;
            BTFSC   PORTB, 0
            BSF     STATUS    , C
            RRF     rxbuff  , f
            DECFSZ  bitcount  , f         ;test if all done
            GOTO    Next_RcvBit
            CALL    Bit_Delay           ; wait for stop bit
            BSF     INTCON,  INTE       ; restore RB0 IRQ for next char
            BCF     COMMSFLAG, RXIP     ; clear comms-in-progress bit
            ;BSF     COMMSFLAG, 0        ; set "rx-done" flag 
            MOVF    rxbuff, W           ; pick up the sent character
            MOVWF   rxbuff2             ; copy to safe place in case another byte arrives
            RETURN                      ; exit with char in W

; Often you wish to simply sit in a loop and wait for a specified period of time.
; Here is a delay subroutine which will give up to 1 millisecond delay for a 4 Mhz clock
; and does not use a file register: 

;MSEC1       MOVLW   0xF9                ; 249 allow for 4 microsec overhead.. 
;            GOTO    MICRO4              ; (2 for CALL)
Start_Delay MOVLW   0x08                ; 8*4+overhead = 32uS (allows <70uS slack)
            GOTO    MICRO4
Bit_Delay   MOVLW   0x18                ; full bit 104uS
            goto    MICRO4
Delay_4m    movlw   0x01                ; 4 microsecond delay
MICRO4      ADDLW   0xFF                ; subtract 1 from W
            ; add 16 NOP for a 20MHz crystal!
            NOP                         ; 4uSec = 20 instruction cycles
            NOP
            NOP
            NOP
            NOP
            NOP
            NOP
            NOP
            NOP
            NOP
            NOP
            NOP
            NOP
            NOP
            NOP
            NOP
            BTFSS   STATUS,Z            ; skip when you reach zero
            GOTO    MICRO4              ; more loops
            RETURN
; Some comments about the code: 
; Each loop takes 4 microseconds or 20 clocks; ADDLW takes 1, BTFSS takes 1 and
; each NOP takes 1 and GOTO takes 2 clocks.
; When the skip is taken it is also 4. 
; SUBLW 1 subtracts W from one, not the reverse. To subract one from W you add the
; two's compliment of 1 which is $FF. 
; You are testing the zero bit, (Z) in the STATUS register which will be set when the
; subtraction results in zero. 
; The bit test takes 1 clock unless the skip is taken, in which case it takes 2. 
; You could call MICRO4 directly with the number of 4 microsec loops in W for delays in
; multiples of 4 microseconds. Remember that the call itself is 2 microseconds.
; Don't enter with 0 in W or you get a 256*4 microsec delay. 
; Notice that the subroutine does not use a register, just 'W'. 


;  ****   SERIAL TRANSMIT *******
; To output a character string from program memory, you set up a register to point to the
; first character, MSGPTR, and repeatedly call "messages", incrementing the pointer each time.
; We have reached the end of the string when a zero is returned.
; The routine is entered with the offset of the first character in 'W':

; These two routines are not yet used....
 
;OUTMSG      MOVWF   MSGPTR              ; put 'W' into message pointer
;MSGLOOP     MOVF    MSGPTR, W           ; put the offset in 'W'
;            CALL    messages            ; returns ASCII character in 'W'
;            ADDLW   0                   ; sets the zero flag if W = 0
;            BTFSC   STATUS, Z           ; skip if zero bit not set
;            RETURN                      ; finished if W = 0
;            CALL    TrX_byte            ; output the character
;            INCF    MSGPTR, f           ; point at next
;            GOTO    MSGLOOP             ; more characters

; send (TX) string of fixed or variable length. XREG has count, calc4 is assumed location.

;OUTbuf8     MOVLW   0x08                ; or set up 8 characters using calc4
;OUTbufX     MOVWF   XREG                ; requested count in XREG using calc4
;            ; XREG has count of RAM characters to send
;MSGLOOP1    MOVLW   calc4             ; set up indirect addressing using calc4
;            ; XREG has count, W is location in RAM (can be varied) i.e. at calc6
;OUTbufW     MOVF    FSR ,  F            ; to point to scratchpad ram 
;MSGLOOP2    MOVF    INDF,  W            ; put the char in 'W'
;            DECFSZ  XREG,  F            ; countdown until 0
;            RETURN                      ; finished if count = 0
;            CALL    TrX_byte            ; output the character in W and update encoders
;            INCF    FSR, f              ; point at ...
;            GOTO    MSGLOOP2            ; more characters


; Encoder processing code **********************************

;       ENCODER COUNTER
;            _____
;       ____|     |_____
;          _____      __
;       __|     |____|
;       0  1  3  2  0
;       0  2  3  1  0
;
;       if encoder was:    0  1  2  3  00 80 10 18     00 02 04 06
;       upcount should be: 1  3  0  2  80 18 00 10     02 06 00 04
;       downcount  is:     2  0  3  1  10 00 18 80     04 00 06 02
;       skipped error:     3  2  1  0  18 10 08 00     06 04 02 01
;

update_encoders
            MOVLW   0x18               ; read RA encoder
            ANDWF   PORTB     , W
            MOVWF   RAenc              ; copy to RAenc
            MOVLW   0x06
            ANDWF   PORTB     , W      ; read DEC encoder
            MOVWF   DECenc             ; copy to DECenc
            SUBWF   oldDECen  , W      ; W = oldDECen - W, oldDECen-DECenc
            BTFSS   STATUS    , Z      ; is W==0? (same as old value)
            CALL    DEC_change         ; no, DEC changed
            MOVF    RAenc  , W         ; yes (same) W=RAenc
            SUBWF   oldRAenc  , W      ; W = oldRAenc-RAenc
            BTFSS   STATUS    , Z      ; test if oldRAenc=RAenc
            CALL    RA_change          ; no, RA changed
            MOVF    DECenc    , W      ; yes same 
            MOVWF   oldDECen           ; oldDECen=DECenc
            MOVF    RAenc  , W
            MOVWF   oldRAenc           ; oldRAenc=RAenc
            RETURN
DEC_change
            MOVF    oldDECen  , W      ; DEC changed! W = oldDECen   0110 0100 0010 0000
            XORWF   DECenc    , W      ; XOR oldDECenc with DECenc   0100           0100
            SUBLW   0x06               ; 6-W test if B chan is same
            BTFSC   STATUS    , Z      ; skip not zero
            GOTO    Log_Error         ; 
            RLF     DECenc    , W      ; W = DECenc >> 1
            XORWF   oldDECen  , f      ; oldDECenc = W XOR oldDECenc
            BTFSC   oldDECen  , 02     ; test chan A???
            GOTO    bump_DEC           ; incr counter
            MOVF    DECcount  , f      ; test counter for underflow
            BTFSS   STATUS    , Z
            GOTO    Label_0010         ; okay to decr if not zero
            MOVF    DECcountHi, f      ; check high byte
            BTFSS   STATUS    , Z
            GOTO    Label_0011         ; decr if not zero
            MOVF    DECmax      , W      ; else copy max count
            MOVWF   DECcount
            MOVF    DECmaxHi      , W
            MOVWF   DECcountHi
            MOVF    DECcount  , f
            BTFSC   STATUS    , Z
Label_0011  DECF    DECcountHi, f      ; and decr counter
Label_0010  DECF    DECcount  , f
            RETURN
bump_DEC
            INCF    DECcount  , f      ; incr counter
            BTFSC   STATUS    , Z      ; ff -> 00?
            INCF    DECcountHi, f      ; yes, inc high byte
            MOVF    DECmax      , W      ; compare with max count
            SUBWF   DECcount  , W
            BTFSS   STATUS    , Z      ; low bytes equal?
            RETURN                     ; not yet
            MOVF    DECmaxHi      , W      ; if yes, high byte must be equal too
            SUBWF   DECcountHi, W      ; test high bytes
            BTFSS   STATUS    , Z      ; if equal
            RETURN
            CLRF    DECcount               ; put count at zero
            CLRF    DECcountHi
            RETURN
RA_change
            MOVF    oldRAenc  , W       ; RAencoder tests
            XORWF   RAenc  , W          ; check against reading for the direction
            SUBLW   0x18
            BTFSC   STATUS    , Z
            GOTO    Log_Error
            RLF     RAenc  , W
            XORWF   oldRAenc  , f
            BTFSC   oldRAenc  , 04
            GOTO    bump_RA             ; add to RA counter
            MOVF    RAcount   , f       ; else decr RA counter
            BTFSS   STATUS    , Z       ; check low byte
            GOTO    Label_0013          ; if non zero, just decr and exit
            MOVF    RAcountHi      , f  ; else high byte has to be decr too
            BTFSS   STATUS    , Z       ; if both are zero, do rollover 
            GOTO    Label_0014          ; else decr both
            MOVF    RAmax     , W       ; underflow
            MOVWF   RAcount             ; put max count into RA counter
            MOVF    RAmaxHi   , W
            MOVWF   RAcountHi
            MOVF    RAcount   , f
            BTFSC   STATUS    , Z
Label_0014  DECF    RAcountHi , f
Label_0013  DECF    RAcount   , f
            RETURN
bump_RA
            INCF    RAcount   , f      ; increment RA counter
            BTFSC   STATUS    , Z
            INCF    RAcountHi , f
            MOVF    RAmax     , W
            SUBWF   RAcount   , W
            BTFSS   STATUS    , Z
            RETURN
            MOVF    RAmaxHi   , W
            SUBWF   RAcountHi , W
            BTFSS   STATUS    , Z      ; check rollover
            RETURN
            CLRF    RAcount            ; count =0
            CLRF    RAcountHi
            RETURN

Log_Error   INCF    ERRcount  , f      ; log error
            RETURN

;set_middle                              ; set DEC & RA encoders to mid position
;            MOVF    DECmax       , W
;            MOVWF   DECcount
;            MOVF    DECmaxHi     , W
;            MOVWF   DECcountHi
;            BCF     STATUS    , C
;            RRF     DECcountHi   , f
;            RRF     DECcount     , f
set_RAmid   MOVF    RAmax      , W      ; set RA encoders to mid position
            MOVWF   RAcount
            MOVF    RAmaxHi      , W
            MOVWF   RAcountHi
            BCF     STATUS    , C
            RRF     RAcountHi    , f
            RRF     RAcount      , f
            RETURN

set_DECLAT  MOVF    LATcount     , W    ; set dec encoders to lattitude
            MOVWF   DECcount
            MOVF    LATcountHi   , W
            MOVWF   DECcountHi
            RETURN

set_LATDEC  MOVF    DECcount     , W    ; set latitude to dec encoder reading
            MOVWF   LATcount
            MOVF    DECcountHi   , W
            MOVWF   LATcountHi
            RETURN

; EEPROM read, write code  *********************************
; EEPROM save routine: save stopwatch, setpoint, maxcounts or the usual suspects

save_LAP                           ; save stopwatch buffer to EEPROM
            movlw   0x30                ;  48 vars
            movwf   XREG
            movlw   0xAA                ; recbuf!  ; offset to vars in FSR
            movwf   FSR                 ; set up indirect adressing
            ; address is 32 + 8pointnum*5 = 72 = $44 
            movlw   0x44                ; byte 32 eeprom address + 8*5 setpoints
            movwf   MSGPTR
next_EELAP  movf    MSGPTR, w           ; pick up address to save in EEPROM
            call    save_EEbyte
            incf    FSR, f
            incf    MSGPTR, f
            decfsz  XREG, f
            goto    next_EELAP
            return



save_Setpoint                           ; save encoder counts to eeprom
            movlw   0x05                ;  5 vars
            movwf   XREG
            movlw   DECcount            ; offset to vars in FSR
            movwf   FSR                 ; set up indirect adressing
            ; address is $20 + pointnum*5
            movf    pointnum, W
            addwf   pointnum, W
            addwf   pointnum, W
            addwf   pointnum, W
            addwf   pointnum, W
           ;addwf   pointnum, W
            ;movwf   MSGPTR
            ;rlf     MSGPTR, f           ; mult by 2
            ;movf    MSGPTR, w
            ;rlf     MSGPTR, f
            ;addwf   MSGPTR, w
            addlw   0x20                ; byte 32 eeprom address + 5 per setpoint
            goto    set_eeadr           ; jump to save loop with our values

save_EEPROMmax                          ; save encoder max counts to eeprom
            movlw   0x04                ; 4 vars
            movwf   XREG
            movlw   DECmax              ; offset to vars in FSR
            movwf   FSR                 ; set up indirect adressing
            movlw   0x01                ; byte 01..05 eeprom address
            goto    set_eeadr           ; jump to save loop with our values

            ; EEPROM save all, or only encoders
save_EEPROM
            movlw   0x20                ; save 32 (was 17) vars to eeprom
ee_vars     movwf   XREG
            movlw   pointnum             ; offset to vars in FSR
            movwf   FSR                 ; set up indirect adressing
            movlw   0x00                ; byte 00..1F eeprom address
set_eeadr   movwf   MSGPTR
next_EE     movf    MSGPTR, w           ; pick up address to save in EEPROM
            call    save_EEbyte
            incf    FSR, f
            incf    MSGPTR, f
            decfsz  XREG, f
            goto    next_EE
            return

save_EEbyte                             ; enter with W=address to write
            BCF     INTCON    , GIE     ; clear GIE, prevent all interupt vectors
            BSF     STATUS    , RP0     ; bank 1   DIFFERENT TO 16F84!
            MOVWF   EEADR               ; eeprom address = W
            MOVF    INDF     , W
            MOVWF   EEDATA              ; eeprom data = variable to save
            CLRF    EECON1
            BSF     EECON1    , 02
            MOVLW   0x55
            MOVWF   EECON2
            MOVLW   0xAA
            MOVWF   EECON2
            BSF     EECON1    , 01
busy_EE     BTFSC   EECON1    , 01      ; could take 4ms (16F84A)
            GOTO    busy_EE
            BCF     EECON1    , 04
            BCF     STATUS    , RP0     ; bank 0
            BSF     INTCON    , GIE    ; restore interrupts
            RETURN

; copy eeprom data to variables: read a setpoint or EEPROM vars

read_Setpoint                           ; should be followed by convert RAhms-HAhms=SIDhms
            call    read_EEPT           ; get the point?
            goto    calc_SIDhms         ; find new SID = RA-HA

read_EEPT   movlw   0x05                ;  5 vars
            movwf   XREG                ; count of vars to do
            movlw   DECcount            ; offset to target vars in FSR
            movwf   FSR                 ; set up indirect adressing
            ; address is $20 + pointnum*5
            movf    pointnum, W
            addwf   pointnum, W
            addwf   pointnum, W
            addwf   pointnum, W
            addwf   pointnum, W
          ; addwf   pointnum, W
            addlw   0x20                ; byte 32+pt*5.. eeprom address
            movwf   MSGPTR              ; temp place to save address
            goto    next_readEE         ; and jump to it

read_EEPROM
            movlw   0x20                ; 32 (was 17) vars
            movwf   XREG                ; count of vars to do
            movlw   pointnum             ; offset to target vars in FSR
            movwf   FSR                 ; set up indirect adressing
            movlw   0x00                ; byte 00..11 eeprom address
            movwf   MSGPTR              ; temp place to save address
next_readEE movf    MSGPTR, w           ; pick up address to read (dest in bank 0)
            bsf     STATUS    , RP0     ; Bank 1  DIFFERENT TO 16F84!
            movwf   EEADR
            bsf     EECON1    , 00
            movf    EEDATA    , W
            bcf     STATUS    , RP0     ; Bank 0
            movwf   INDF                ; put it in target
            incf    FSR, f              ; next target var
            incf    MSGPTR, f           ; next eeprom address
            decfsz  XREG, f             ; loop finished?
            goto    next_readEE
            return

;  ****************  INTERRUPT CODE  *****************************
; Interrupt processing code
; We handle four interrupts at once
;    Stopwatch counter (timer2)
;    7seg display multiplexer (timer1)
;    Serial communications (RB0)
;    Real Time Clocks (Normal + Siderial) (timer0)

; Interupt - first, save W, status (assume using FSR in irq)

IRQ_Start   MOVWF   SavedWR             ; saved W
            SWAPF   STATUS    , W
            MOVWF   SavedSTAT           ; saved STATUS
            MOVF    FSR, W
            MOVWF   SavedFSR            ; save FSR
            movf    PCLATH,w
            movwf   SavedPCLATH
            clrf    PCLATH
            bcf     STATUS,  RP0        ; set for bank 0 during IRQ
; test for interrupt type: timer1, Timer2, RB0, timer0
check_TMR2  btfsc   PIR1      , TMR2IF  ; check irq flag for timer 2
            goto    timer2_IRQ          ; process timer2 lap/stopwatch
check_TMR1  btfsc   PIR1      , TMR1IF  ; check irq flag timer 1
            goto    timer1_IRQ          ; process timer1 7seg display updater
check_RB0   BTFSC   INTCON    , INTE    ; RB0 external interrupt enabled?
            GOTO    Serial_IRQ          ; check RBO irq flag first
check_TMR0  BTFSC   INTCON    , T0IF    ; timer0 interrupt fired? (every 13mS)
            GOTO    Timer_IRQ
; otherwise not our problem , exit interrupt...
close_IRQ   MOVF    SavedFSR, W
            MOVWF   FSR                 ; restore FSR
            movf    SavedPCLATH, w
            movwf   PCLATH              ; restore PCLATH
            SWAPF   SavedSTAT      , W
            MOVWF   STATUS              ; restore status
            SWAPF   SavedWR      , f
            SWAPF   SavedWR      , W    ; restore W
            RETFIE

; Timer 2 used to update stopwatch
; Original code: make lap0 count by tens and update lap1/lap2 at 100 hz.
; Max count is 655.35 seconds

timer2_IRQ  
            bcf     PIR1,  TMR2IF       ; clear the timer2 irq flag
            bsf     STATUS    , RP0     ; ***************  Bank 1 RAM
            incfsz  lap0, f             ; increment the stopwatch counter
            goto    tm2_done            ; exit if not FF->00
            movlw   0xF6                ; tenth count so make it -10
            movwf   lap0
            incfsz  lap1, f             ; 2 byte increment
            goto    tm2_done
            incfsz  lap2, f             ; 3 byte increment
            goto    tm2_done
            incf    lap3, f             ; 4 byte increment
tm2_done    bcf     STATUS    , RP0     ; ***************  Bank 0
            goto   check_TMR1           ; check next interupt...

; Timer 1 used to update 7 seg display digit

timer1_IRQ  
            bcf     PIR1      , TMR1IF  ; clear the timer1 irq flag
            movlw   0xE0                ; 8192 counts = 1.6384 mSec
            movwf   TMR1H               ; set high byte to 224 for next rollover
            movf    dignum, w           ; get digit to do  0-11
            addlw   buff0               ; find offset in buffer
            movwf   FSR
            movf    INDF, w             ; get the seg code
            call    led_segs            ; from look up table
            call    led_shifter         ; and send serially to 74HC595 registers
            movf    dignum, w           ; get digit again  0-11
            call    led_addr            ; get address pattern
            call    led_shift_go        ; send serially to 74HC595 registers
                                        ; and latch the outputs
            incf    dignum, f           ; next digit
            btfss   dignum, 3           ; is it at least 8?
            goto    check_RB0           ; no, next please
            btfsc   dignum, 2           ; is it 12?
            clrf    dignum              ; yes, reset to first digit
            goto    check_RB0           ; finished

; LED port   3 wire interface to 12 x 7seg LED via 74HC595 shifter
;      RB7 data     shifts segment data and digit address to two 74HC595
;      RB6 enable   latch outputs when all bits shifted in place
;      RB5 clock    pulse clock for each bit
; To send address or data to the LED segs, you disable the address/decoder latch,
; then send the bits one at a time, and toggle the clock for each bit.
; After 16 bits, pulse the enable latch.

led_shifter                             ; send 8 bits to 74HC595 registers
            movwf   lcd_tmp             ; save data
            bcf     PORTB, 6            ; make sure latch is low
            movlw   8                   ; 8 bits
            movwf   lcd_cnt
ledloop
            rlf     lcd_tmp, f          ; shift high bit to carry
            btfss   status, C
            goto    carryclear
            bsf     PORTB, 7            ; if 1, set data line
            goto    nextshift
carryclear
            bcf     PORTB, 7            ; if 0, clear data line
nextshift
            nop
            nop                         ; make sure data has time to settle...
            nop
            bsf     PORTB, 5            ; toggle the clock
            nop
            nop                         ; stretch it to 1 microsecond...
            nop
            bcf     PORTB, 5
            decfsz  lcd_cnt, f          ; check if all bits shifted
            goto    ledloop
            return

led_shift_go
            call    led_shifter         ; shift the bits
            bsf     PORTB, 6            ; latch pulse for shift reg
            nop
            nop                         ; stretch it to 1 microsecond...
            nop
            bcf     PORTB, 6
            return


; external interrupt, RB0  - serial rs232 receiver
; This interrupt is enabled at idle times but disabled during rs232 receive
; so that it is not triggered by bit swiches during reception.
; if we arrive here, we must check the flag and return if it has not fired.
; If it has fired, we check first for a glitch (paranoia)

Serial_IRQ  BTFSS   INTCON    , INTF    ; skip over exit if flag set
            GOTO    check_TMR0          ; ignore if flag is clear
            BCF     INTCON    , INTF    ; acknowlege, clear RB0 int flag
            BTFSC   PORTB     , 00      ; test RB0==0   still?
            GOTO    check_TMR0          ; RB0=1, so restore & exit. glitch?

            BSF     COMMSFLAG, RXIP     ; flag=2 Log RX incoming bit 1
            BCF     INTCON , INTE       ; disable RB0 interrupt during reception
            GOTO    check_TMR0          ; next please

; main code should check soon for incoming rx data and restore RB0 interrupt when done.

; Timer0 overflow interrupt
; Update clock every 65.536/5 = 13.1mSec

Timer_IRQ   BCF     INTCON    , T0IF    ; clear timer0 overflow flag
;           bcf     COMMSFLAG , 6       ; clear last secounds update irq flash bit
            BTFSS   COMMSFLAG ,  RXIP   ; test bit 2 for rs232 in progress
            GOTO    Adjust_clock        ; clear, normal clock update
                                        ; set,   just do a quick IRQ (tidy up next time)

Quick_Adjust
	; add 65536 counts to the microsecond counters (what an easy thing to do!)
	incf	u2,f
        incf    st2,f                   ; bump sid counter too
        goto  close_IRQ                 ; bail out

Adjust_clock
	; add 65536 counts to the microsecond counter. This is done every 13.1 mS
	incf	u2,f
	
	; save a copy of the 24 bit counter
	movf    u0, w                   ; low byte
	movwf   t0
	movf    u1, w                   ; middle
	movwf   t1
	movf    u2, w                   ; top
	movwf   t2                 	;

        ; for normal time, update time (seconds) every 1 million uSec
        ; This is 5 million counts for a 20MHz clock, or 1 mill for a 4MHz clock.
        ; this should keep good time if the 20MHz crystal is accurate, < 0.1 sec per day

        ; subtract 1s from copy of counter  (change: 5 million counts = 1 sec for 20MHz clock)

	movlw	d'5000000'&0xff
	subwf	t0,f	
	btfss	status,c
	decf	t1,f	
	
	movlw	d'5000000'>>d'8'&0xff
	subwf	t1,f	
	btfss	status,c
	decf	t2,f	
	
	movlw	d'5000000'>>d'16'&0xff
	subwf	t2,f	
	btfss	status,c
	goto	Adjust_sidclock         ; underflow, ignore the subtraction, do sid clock.

	;; did not underflow, save the result, occurs once per second
	movf    t0, w                   ; low byte
	movwf   u0
	movf    t1, w                   ; middle
	movwf   u1
	movf    t2, w                   ; top
	movwf   u2                 	;

        ; toggle the blink bit each second
        btfsc   COMMSFLAG, 7
        goto    c_blink
        bsf     COMMSFLAG, 7        ; set blink bit
        goto    one_sec
c_blink bcf     COMMSFLAG , 7       ; clear blink bit

one_sec        
;       bsf     COMMSFLAG , 6

	; point to counter   - time of day
	movlw	secs                    ; update normal time
        call    update_clkW
        goto    Adjust_sidclock         ; update siderial time

; check for extra sid update use ratio of 1461:1465
; here I try to do both: use 1 million uSec update and a 1461/1465 ratio hack
; to add extra ticks to the siderial clock. See below for grisly details.
; I don't have the registers for another microsecond counter in a 16F84 unless I
; delete the normal clock and I want both clocks, hence the hack.

; For the 16F628A, we have the room, so the hack is deleted.
; This frees two vars: sidcount and sidcountHi for other uses.

         ; Hugo's famous update siderial counter hack
;        incfsz  sidcount, f             ; skip if ff>0 - do the low byte
;        goto    close_IRQ               ; exit if not ff>00
;        incfsz  sidcountHi, f           ; then the hi byte
;        goto    inc_test                ; check for extra clocks FB..FE
;        movlw   0xFA                    ; FF>00 so restart sid counter
;        movwf   sidcountHi              ; 
;        movlw   0x4B                    ; 0xFA4B is -1461
;        movwf   sidcount                ; 
;        goto    close_IRQ               ; exit

;inc_test                                ; check for extra sid ticks
;        movlw   0xFF
;        subwf   sidcountHi, w           ; not if FF
;        btfsc   STATUS, Z               
;        goto    close_IRQ
;        movlw   sidsec                  
;        call    update_clkW             ; extra siderial clock tick

;        goto    close_IRQ

update_clkW                             ; update clock HHMMSS pointed to by W reg
	movwf	fsr

	incf	indf,f			; increment second counter
	
	movlw	d'60'
	subwf	indf,w
	btfss   STATUS, Z               ; bnz isr_kick_end ; hasn't reached a minute
        return

	clrf	indf			; clear the second counter
	incf	fsr,f			; point to minute counter
	incf	indf,f			; increment minute counter
	
	movlw	d'60'
	subwf	indf,w
        btfss   STATUS, Z
	return                          ; hasn't reached an hour

	clrf	indf			; clear the minute counter
	incf	fsr,f			; point to hour counter
	incf	indf,f			; increment hour counter

	movlw	d'24'
	subwf	indf,w
        btfss   STATUS, Z
	return                          ; hasn't reached 24 hours

	clrf	indf			; clear the hour counter
        return

Adjust_sidclock
	; add 65536 counts to the microsecond counter. This is done every 13.1 mS
	incf	st2,f
	
	; save a copy of the 24 bit siderial counter
	movf    st0, w                   ; low byte
	movwf   t0
	movf    st1, w                   ; middle
	movwf   t1
	movf    st2, w                   ; top
	movwf   t2                 	;

        ; for siderial time, update sid seconds every 997270 uSec, ratio 1.00273795,
        ; i.e. 4986348 counts at 20MHz clock, 997270 otherwise
        ; this should be correct to 1 in a million, or < 0.1 sec per day

        ; subtract .997270 sec from copy of sid counter  (4986348 for 20MHz clock)

	movlw	d'4986348'&0xff
	subwf	t0,f	
	btfss	status,c
	decf	t1,f	
	
	movlw	d'4986348'>>d'8'&0xff
	subwf	t1,f	
	btfss	status,c
	decf	t2,f	
	
	movlw	d'4986348'>>d'16'&0xff
	subwf	t2,f	
	btfss	status,c
	goto	close_IRQ               ; underflow, ignore the subtraction

	;; did not underflow, save the result, occurs once per second
	movf    t0, w                   ; low byte     ; mov24	u0,t0
	movwf   st0
	movf    t1, w                   ; middle
	movwf   st1
	movf    t2, w                   ; top
	movwf   st2                 	;

	; point to counter   - siderial time
        movlw   sidsec                  ; update siderial time
        call    update_clkW
	goto	close_IRQ               ; back to main IRQ code



; lcd interface code ************ REMOVED ***************************
; lcd display gave too many problems with write delays and rs232 code...
; so we can remove the delay code since it is not used anywhere else.
;#define LCD_CK	PORTB, 5
;#define LCD_DA	PORTB, 6
;#define LCD_E	PORTB, 7

;lcd_delay160us	movlw 0x28              ; 40, was $36 ; calibrated for 4 MHz
;                call  MICRO4            ; 4 uSec delay
;
;lcd_delay5ms	movlw 0x20              ; for init and clear display command
;		movwf lcd_dly2
;lcd_delay1	call lcd_delay160us
;		decfsz lcd_dly2, F
;		goto lcd_delay1
;		return

; Mathematics code *******************************************

;                           MUL8x8.ASM
;                   8x8 Software Multiplier
;
;   The 16 bit result is stored in 2 bytes
;
; Before calling the subroutine "multiply8x8", the multiplier should
; be loaded in location "mulplr", and the multiplicand in
; "mulcnd".  The 16 bit result is stored in locations
; H_byte & L_byte.
;
;       Performance :
;                       Program Memory  :  14 locations
;                       # of cycles     :  71
;                       RAM             :   5 locations
;
;  This routine is optimized for code efficiency ( looped code )
;  For time efficiency code refer to "mult8x8F.asm" ( straight line code )
;*******************************************************************
; This, and the following macros came from a web search, I forget who did it...

mulcnd  EQU     0x78    ; 8 bit multiplicand
mulplr  EQU     0x79    ; 8 bit multiplier
H_byte  EQU     0x7A    ; High byte of the 16 bit result
L_byte  EQU     0x7B    ; Low byte of the 16 bit result
countM  EQU     0x7C    ; loop counter

multiply8x8             ; we do not use this, here for interest sake...

;        clrf    H_byte
;        clrf    L_byte
;        movlw   8
;        movwf   countM
;        movf    mulcnd, W
;        bcf     STATUS, C        ; Clear the carry bit in the status Reg.
;loop    rrf     mulplr, F
;        btfsc   STATUS, C
;        addwf   H_byte, F
;        rrf     H_byte, F
;        rrf     L_byte, F
;        decfsz  countM, F
;        goto    loop

;        retlw   0


;--------------------------------------------------------------------------
; add with carry - adds the w register and the carry flag to the file
; register reg, returns the result in <reg> with the carry flag set if overflow
;--------------------------------------------------------------------------

addcwf    macro reg

          local add1,add2

          bnc add1                      ; branch if no carry set

          addwf reg , f                 ; add byte

          incf reg , f                  ; add carry
          skpnz
          setc

          goto add2

add1      addwf reg,f                   ; add byte

add2
          endm


;--------------------------------------------------------------------------
; subtract with "no-carry" - subtracts the w register and the no-carry flag
; from the file register reg, returns the result in reg with the no carry flag
; set if underflow
;--------------------------------------------------------------------------

subncwf   macro reg

          local sub1,sub2

          bc sub1                       ; branch if carry set
          subwf reg, f                  ; subtract byte
          skpnz                         ; subtract no carry
          clrc
          decf reg , f
          goto sub2
sub1      subwf reg , f                 ; subtract byte
sub2
          endm


;--------------------------------------------------------------------------
; MACRO to perform 32-bit addition - adds the four bytes at op2 to the
; four (three?) bytes at op1 (most significant bytes first), returns the result in
; op1 with op2 unchanged and the carry flag set if overflow
;--------------------------------------------------------------------------

add32     macro op1,op2                 ; op1 := op1 + op2

          movfw op2+3                   ; add low byte        (bits 7...0)
          addwf op1+3,f

          movfw op2+2                   ; add middle-low byte (bits 15..8)
          addcwf op1+2

          movfw op2+1                   ; add middle-high byte (bits 23...16)
          addcwf op1+1

          movfw op2+0                   ; add high byte       (bits 31...24) 
          addcwf op1+0

          endm


;--------------------------------------------------------------------------
; MACRO to perform 32-bit subtraction - subtracts the four bytes at op2
; from the four bytes at op1 (most significant bytes first), returns the
; result in op1 with op2 unchanged and the no carry flag set if underflow
;--------------------------------------------------------------------------

sub32     macro op1,op2                 ; op1 := op1 - op2

          movfw op2+3                   ; subtract low byte 
          subwf op1+3 , f

          movfw op2+2                   ; subtract middle low byte
          subncwf op1+2

          movfw op2+1                   ; subtract middle high byte
          subncwf op1+1

          movfw op2+0                   ; subtract high byte
          subncwf op1+0

          endm


;--------------------------------------------------------------------------
; MACRO to negate a 32-bit value  (  op := 0 - op ) .
;--------------------------------------------------------------------------

neg32  macro op                      ; op1 := 0 - op2
       local neg_done
           comf  op,   f             ; invert all 8 bits in high byte  
           comf  op+1, f             ; invert all 8 bits in middle high byte  
           comf  op+2, f             ; invert all 8 bits in middle low byte  
           comf  op+3, f             ; invert all 8 bits in low byte  
           ; Note at this point 0x000000 would have turned into 0xFFFFFFF .
           ; Must add ONE to complete the TWO's COMPLIMENT calculation ( -0  = 0 ).
           ; Note that "incf" affects only the Z flag but not the C flag .
           incfsz op+3, f            ; increment low byte        (bits 7...0)
           goto   neg_done           ; if incremented result NOT zero, we're through !
           incfsz op+2, f            ; increment middle low byte (bits 15...8)
           goto   neg_done           ; if incremented result NOT zero, ...
           incfsz op+1, f            ; increment middle high byte (bits 23...16)
           goto   neg_done           ; if ...
           incfsz op+0, f            ; increment high byte       (bits 31...24)
           goto   neg_done           ;
neg_done
       endm

; *********************************************************************
; Set Points require saving RA and DEC coords in EEPROM and retreiving them at synch time.
; DEC can be a straight forward save of encoder counts and restoring them later.
; RA is complicated by needing to account for different siderial times;
; The RA can be stored as hh mm ss or a 24 bit number 0-86400.
; From RA, subtract Hour Angle, result is the correct siderial time.
; This assumes that hour angle encoder is set to mid count at startup.

; Normal routine:
; DECencoder -> DECseconds -> DECdms   (conv_DECcount)
; HAencoder  -> HAseconds -> HAhms     (conv_RAcount - HAhms stored in HAhh,HAmm,HAss)
; RAhms = SIDhms + HAhms
; check RAhms for overflow (0-23 59 59)

; Store routine:
; DECencoder -> EEPROM
; RAhms      -> EEPROM

; Retrieve routine:
; EEPROM -> DECencoder
; EEPROM -> RAhms
; HAencoder -> HAseconds -> HAhms   (conv_RAcount)
; SIDhms - HAhms = SIDhms
; check SIDhms for underflow (<0?)
; Note: In retrieving the RA, I change the siderial time instead of the RA encoder count.
; I found that this was less involved than trying to convert RA back to encoder counts,
; mostly because of having to change multiply factors with different encoder max values.

; convertion from counts to hms ; 86400 seconds in 24 hours
; encoder count is in range 0-EncMAX, 0-21962 for example.
; Find a constant 'K' to multiply it by, add factor of 65536 to preserve accuracy
; K = (86400*65536)/EncMAX = 257823 $03EF1F  This will change for different EncMAX.
; Multiply RAcount * 257823 and discard last two bytes. The result is a 24 bit number
; in the range 0-86400. Then use successive subtraction to convert to hh mm ss.

temp0     EQU   calc0       ; copy of "count"
temp1     EQU   calc1
fact4     EQU   calc2       ; space to rotate factor by 16 bits
fact3     EQU   calc3       ; 
fact2     EQU   calc4       ; 0x1F
fact1     EQU   calc5       ; 0xEF
fact0     EQU   calc6       ; 0x03
dest4     EQU   calc7       ; 24 bit answer
dest3     EQU   calc8       ;
dest2     EQU   calc9       ;
dest1     EQU   calc10      ; destination, discard low bytes
dest0     EQU   calc11      ;
; uses XREG as counter


; converting counts to HA hh mm ss    
; 1 convert counts to range 0-86400   
; 2 convert 0-86400 to HH MM SS       
; also need sid time + hr angle = ra hms
; after synch: Sidt = ra h m s - hr angle OR hr angle = RAhms - Sidt

; multiply RAcount by magic number ($03EF1F) to get HA in seconds
; this number is 864000*65536/RAencMAX (changes with encoder max value)
conv_RAcount                    
        movf    RAcount, w
        movwf   temp0
        movf    RAcountHi, w
        movwf   temp1
        movf    HAfact0, w          ; 0x1F
        movwf   fact0
        movf    HAfact1, w          ; 0xEF
        movwf   fact1
        movf    HAfact2, w          ; 0x03
        movwf   fact2
        call    mult16              ; result in dest4,3,2   0,1 discard
        MOVF    dest2  , W
        MOVWF   lcd_dly1
        MOVF    dest3, W
        goto    con_hms24           ; conv->HAhh HAmm HAss

; similarly for DECcount and DECfact0..2 where factor is (say) $15AEB1    1,420,977 4981
; i.e. 355mm/5.7mm*160=9962, half is 4981 (180 deg), 108000*65536/4981=1,420,977
; we need to account for changing scales at equator at an encoder count of 9962/4
; equator count = 2490.5, equivalent to 'seconds'=54000 (0xD2F0)
; equator count + 1/2 (2491), gives 90d 01.1 (offset from south pole)
; If we subtract 90 from the 'degrees' column we find the correct answer = +0d 01.1
; equator count - 1/2 (2490), gives 89d 58.9 (offset from south pole)
; If we subtract 90 from the 'degrees' column we find the wrong answer (89 - 90 = -1)
; or -1d 58.9:   BUT THIS SHOULD BE -0d 01.1
; i.e we can't just subtract 90 deg from the 'degrees' like we can for northern dec.
; So we calculate 'seconds' and test for above or below the equator count.
; We subtract 54000 and if no underflow, convert result to 0-90deg. North Declination.
; if it underflows, we calc 54000-seconds, set a negative flag, and convert the result
; to 0..90 (South) which we display as -dd mm.m

; use some of the calc vars for our convert test

; subtract 54000 from dec
; copy dest2 3 4 to calc0 1 2
; subtract, test overflow
; if high byte is nonzero, result is a southerly dec, so calc 54000-count
; case:       65536 $010000    54001 $D2F1    54000 $00D2F0    53999 $00D2EF    00001 $000001
; sub 54000   11536 $002D10    00001          00000            -1    $FFFFFF   -53999 $FF2D11
; 54000-count                                                  00001 $000001    53999 $00D2EF

dec_NS
	movlw	d'54000'>>d'8'&0xff
        movwf   calc11	                ; save this value in case we need it in south_dec
        ; test for 00 00 bug and go direct to south calc
        movf dest4,w
        andlw 0xFF
        btfss status, z
        goto dec_testNS          ; bail out if non zero
        movf dest3, w
        andlw 0xFF               ; are they both zero?
        btfsc status, z
        goto south_dec

dec_testNS
	; save a copy of the dec counter
	movf    dest2, w                   ; low byte
	movwf   calc0
	movf    dest3, w                   ; middle
	movwf   calc1
	movf    dest4, w                   ; top
	movwf   calc2                 	;

        ; subtract 54000 from copy of counter to test for north/south dec

	movlw	d'54000'&0xff
    
	subwf	calc0,f	
	btfss	status,c
	decf	calc1,f	
	
	movlw	d'54000'>>d'8'&0xff
        subwf	calc1,f	
	btfss	status,c
	decf	calc2,f	
	; now check for $FF in the high byte
        btfsc   calc2, 7
	goto	south_dec               ; underflow, ignore the subtraction, do 54000-dec.

	; NORTH: did not underflow, save the result for conversion routine
	movf    calc0, w                ; low byte
	movwf   dest2
	movf    calc1, w                ; middle
	movwf   dest3
	movf    calc2, w                ; top
	movwf   dest4                 	
                                        ; set positive flag
        bcf     decsign, 7
        return

south_dec                       
	; 0-53999: get original copy of the dec counter
	movf    dest2, w                   ; low byte
	movwf   calc0
	movf    dest3, w                   ; middle
	movwf   calc1
	movf    dest4, w                   ; top
	movwf   calc2

        ; subtract counter from 54000  i.e 54000-count

	movf	calc0, w                ; low byte
	sublw	d'54000'&0xff           ; subtract
        movwf   dest2	
	btfss	status,c
	decf	calc11,f	
	
	movf	calc1, w                ; mid byte
        subwf   calc11,w	
        movwf   dest3	
	btfss	status,c
	decf	calc2,f	                ; should not happen
	
	movlw	d'0'
        movwf   dest4	
                                        ; set negative flag
        bsf     decsign, 7
        return



conv_DECcount
        movf    DECcount, w
        movwf   temp0
        movf    DECcountHi, w
        movwf   temp1
        movf    DECfact0, w            ; 0xB1
        movwf   fact0           
        movf    DECfact1, w            ; 0xAE
        movwf   fact1
        movf    DECfact2, w            ; 0x15
        movwf   fact2
        call    mult16                ; result in dest4,3,2   0,1 discard
        ; now we adjust to account for north/south declination conversion to dd mm.m
        ; we need 24bit subtraction code
        ; subtract 54000 and test for underflow
        call    dec_NS                ; result is 0-53999 with sign bit set appropriately
        MOVF    dest2  , W
        MOVWF   lcd_dly1
        MOVF    dest3, W
        goto    con_dec24             ; conv->dd mm.s and return

; multi byte 16bit multiply required by encoder conversions, 24/32bit * 16bit

mult16                                ; 16 mult required by encoder stuff 
        clrf    dest0     
        clrf    dest1     
        clrf    dest2     
        clrf    dest3     
        clrf    dest4     
        clrf    fact3     
        clrf    fact4
        movlw   0x10                  ; 16 bit
        movwf   XREG
loop16  bcf     STATUS, C             ; test each bit in number
        rrf     temp1, f
        rrf     temp0, f
        btfsc   STATUS, C
        call    add_32cc              ; add fact to dest if bit is 1
        bcf     STATUS, C
        rlf     fact0, f              ; mult factor by 2 for next bit test
        rlf     fact1, f
        rlf     fact2, f
        rlf     fact3, f
        rlf     fact4, f
        decfsz  XREG, f
        goto    loop16
        return

; 40 bit destination (5 byte add) except we ignore the lowest byte since it
; wont amount to much. We need 24bit but 32 bit is used to retain accuracy.
; add_24 and add_16 assume carry is clear before entry                            

add_32cc    bcf     STATUS, F           ; clear carry to start (may not be necessary)
add_32      ; 32 bit add with carry sequence, add fact to dest
            MOVF    fact1      , W          ; start with lowest bytes
            ADDWF   dest1      , f
add_24      MOVF    fact2      , W
            BTFSS   STATUS    , C
            GOTO    add_24cc
            ADDWF   dest2      , f
            INCF    dest2      , f
            BTFSC   STATUS    , Z
            BSF     STATUS    , C
            GOTO    add_16
add_24cc    ADDWF   dest2      , f
add_16      MOVF    fact3      , W
            BTFSS   STATUS    , C
            GOTO    add_16cc
            ADDWF   dest3      , f
            INCF    dest3      , f
            BTFSC   STATUS    , Z
            BSF     STATUS    , C
            GOTO    add_8
add_16cc    ADDWF   dest3      , f
add_8       MOVF    fact4      , W
            BTFSS   STATUS    , C
            GOTO    add_8cc
            ADDWF   dest4      , f
            INCF    dest4      , f
            BTFSC   STATUS    , Z
            BSF     STATUS    , C
            return
add_8cc     ADDWF   dest4      , f
            return


con_hms24                               ; brute force convert 24 bits to h m s
            MOVWF   lcd_dly2            ; 0-86400
            MOVLW   0x0E                ; subtract $E10 3600/hr
            MOVWF   lcd_buf2
            MOVLW   0x10
            MOVWF   lcd_buf1
            CALL    subdigit24
            MOVWF   HAhh
            MOVLW   0x0                 ; subtract 60 $3C  units of mins
            MOVWF   lcd_buf2
            MOVLW   0x3C
            MOVWF   lcd_buf1
            CALL    subdigit16
            MOVWF   HAmm
con_hms08   CLRF    lcd_buf2            ; remainder is secs
            MOVF    lcd_dly1  , W
            MOVWF   HAss
            RETURN

; declination: we assume 360  swing not required     17 words
; -90 to +90 uses 108000 counts   01a5e0
; so I can check for a 1 in the high byte, just like RA
; 1 in high byte = 109 deg. if -90 = 0 then 109 deg = dec of 19 deg

con_dec24                               ; brute force convert 24 bits to dd mm.m
            MOVWF   lcd_dly2            ; 0-54000
            MOVLW   0x02                ; subtract $258 600 hrs $2710 = 10,000  26 words
            MOVWF   lcd_buf2
            MOVLW   0x58
            MOVWF   lcd_buf1
            CALL    subdigitdec
            MOVWF   DECdd
            MOVLW   0x0                 ; subtract 10 $0A  units of mins $03E8 = 1,000
            MOVWF   lcd_buf2
            MOVLW   0x0A      ; was 3C                ; -60
            MOVWF   lcd_buf1
            CALL    subdigit16
            MOVWF   DECmm
            CLRF    lcd_buf2            ; remainder id secs
            MOVF    lcd_dly1  , W
            MOVWF   DECss
            RETURN

subdigitdec  ; test if high byte is 1, if so subtract 108 deg 64800 by adding 736 to dly1,2
            movlw   0x6B                ; 107 = 108-1       35 words
            movwf   XREG
            btfss   dest4,0             ;fact4, 0            ; should be dest4 not fact4
            goto    subdigit16
            goto    add736
subdigit24  ; test if high byte is 1, if so subtract 18hrs 64800 by adding 736 to dly1,2
            movlw   0x11                ; 18-1      
            movwf   XREG
            btfss   dest4,0             ;fact4, 0            ; should be dest4 
            goto    subdigit16
add736                                  ; add 736 2E0
            movlw   0xE0
            ADDWF   lcd_dly1  , f       ; low byte first
            MOVLW   0x02
            BTFSC   STATUS    , C       ; check overflow
            ADDLW   0x01                ; and fix high byte
            ADDWF   lcd_dly2  , f
            goto    digitloop2          ; subdigitX?
subdigit16  MOVLW   0xFF                ; - 1   19 words
subdigitX   MOVWF   XREG                ; dly1,2 is the original count
            goto    digitloop           ; buf1,2 is the number to subtract
            ; part of digit16 subtraction routine


;  RA conversion with siderial time
; Encoder converts to Hour Angle in HAhh HAmm HAss
; Siderial time is in sidhr, sidmin, sidsec
; Find RAhh RAmm RAss by RA = Ha + SID
; NOTE THE FOLLOWING IFs AND BUTs WELL!
; Hour angle is originally set to mid-count at the meridian (12 hr) NOT ZERO HR!
; So 12 needs to be subtracted from the final RA hour
; However 1: if RA < 12, the subtraction will turn out negative, so ADD 12 hr instead.
;         2: for overflows (RA > 24) our figure will be > 36, so we subtract 36 instead.
 
calc_RAhms
            movlw   0x0
            movwf   RAmm
            movwf   RAhh
            movf    HAss, w             ; lowbyte first
            addwf   sidsec, w
            movwf   RAss                ; save, just in case
            movlw   d'60'
            subwf   RAss,w
            btfss   STATUS, C           ; if carry set, its > minute so update RAmm
            goto    calc_RAmin          ; clear,  keep old value
            movwf   RAss
            incf    RAmm, f
calc_RAmin
            movf    RAmm, w
            addwf   HAmm, w             ; minutes
            addwf   sidmin, w
            movwf   RAmm                ; save, just in case
            movlw   d'60'
            subwf   RAmm,w
            btfss   STATUS, C           ; if carry set, its > 60 minute so update RAhh
            goto    calc_RAhour         ; clear,  keep old value
            movwf   RAmm
            incf    RAhh, f
calc_RAhour
            movf    RAhh, w             ; carry
            addwf   HAhh, w             ; add HA + Sid hours
            addwf   sidhr, w
            movwf   RAhh                ; save, just in case
            ; now test if less than 12, greater than 36 or in between
            movlw   d'36'
            subwf   RAhh,w
            btfss   STATUS, C           ; if carry set, its > 36 hours, so sub 36
            goto RAsub12                ; else  check for < 12
            movlw   d'36'
            subwf   RAhh, f             ; sub 36 and save
            return          

RAsub12
            movlw   d'12'
            subwf   RAhh,w
            btfss   STATUS, C           ; if carry set, its > 12 hours, so sub 12
            goto RAadd12                ; else  add 12
            movlw   d'12'
            subwf   RAhh, f             ; sub 12 and save
            return      
            
RAadd12     movlw d'12'
            addwf   RAhh, f
            return    

; calculate sid from ha and ra
; SID = RA - HA  However our HAencoder has a bias of 12 hr that we had to account for above.
; To avoid negative outcomes, we here add a bias of 24 to start and subtract 12 at
; the end. This leaves just one test for sid > 24 to finish. Talk about weird!

calc_SIDhms
            movf    HAss, w
            subwf   RAss, w
            movwf   sidsec
            btfsc   status, c           ; test for overflow
            goto    calc_sidmin         ; cs=no overflow, continue
            addlw   d'60'               ; else add in 60
            movwf   sidsec
            incf    HAmm, f             ; and borrow

calc_sidmin
            movf    HAmm, w
            subwf   RAmm, w
            movwf   sidmin
            btfsc   status, c           ; test for overflow
            goto    calc_sidhour        ; cs=no overflow, continue
            addlw   d'60'               ; else add in 60
            movwf   sidmin
            incf    HAhh, f             ; and borrow

            ; we now need to add our bias because our HAhh is too high by 12
            ; i.e (RA + 24) - HA - 12 = SID, if SID > 24 then subtract 24
            ; note that subwf subtracts w from f, i.e answer = f-w, not the reverse!
calc_sidhour
            movlw   d'24'
            addwf   RAhh, w
            movwf   sidhr
            movf    HAhh, w
            subwf   sidhr, f
            movlw   d'12'
            subwf   sidhr, f
            movlw   d'24'
            subwf   sidhr, w
            btfss   status, c           ; test for overflow
            return                      ; cc = overflow, so must be < 24, exit
                                        ; else keep the subtraction
            movwf   sidhr 
            return


; ******************* NAVIGATION STAR CODE ***************************
; format RAhh,mm,ss,DECdd,mm,ss,0, "name",0  
; Dec is HEX format, 65536=360, decss = 10 (north) or 80 (south)
;   hh mm ss dd mm ss 0 n1 n2 n3 n4 n5 n6 n7 n8 0

get_nav_name    call    get_nav_addr    ; navstar in w, returns offset in w
                addlw   0x07            ; add offset to name
                movwf   ptr
nam_strout	; REM ptr at name[0]
                movlw    calc4
                movwf    FSR
nam_00
                movlw    0x07
                movwf    PCLATH
		movf     ptr, W
		call     navigation
		andlw    0xff           ; set zero flag?
		bz nam_01
		movwf    INDF           ; output char in W to buffer
                incf     FSR, f         ; next destination
		incf     ptr, F             ; next char
		goto     nam_00
nam_01          clrf     PCLATH
		return

get_nav_addr    movf    navstar, w
                movwf   ptr
                bcf     status, c       ; multiply by 16
                rlf     ptr, f          ; mult by 2
                rlf     ptr, f          ; mult by 2
                rlf     ptr, f          ; mult by 2
                rlf     ptr, w          ; mult by 2
                addlw   0x01            ; offset to RAhh byte
                return

get_navstar     call    get_nav_addr    ; offset in w to RAhh byte

nav_strout	movwf    ptr            ; source offset
                movlw    RAhh           ; destination RAhh....DECss, 6 bytes
                movwf    FSR
                movlw    0x07
                movwf    PCLATH
nav_00		movf     ptr, W
		call     navigation     ; get data and test if 6 bytes + 0 done
		andlw   0xff            ; set zero flag?
		bz nav_01
		movwf    INDF           ; output char in W to buffer
                incf     FSR, f         ; next destination
		incf     ptr, F         ; next char
		goto     nav_00

nav_01          clrf     PCLATH
                ; RAhh.mm.ss has the new RA, DECdd.mm has DEC in 65536/360deg format (-90=0).
                ; convert RA to new siderial time (saves messy convertion to encoder counts),
                ; convert DECddmm to encoder counts
                ; DECss was $80 south or $10 north, now we don't use it at all.
                clrf     DECss          ; fix up DEC ss  (probably not required)
                call    calc_SIDhms     ; find new SID = RA-HA
                goto     dectocount     ; dec ddmm to encoder counts and return


; Navigation/Alignment stars in ROM: we have to convert Dec bytes to encoder counts...
; After this, the normal encoder to display routines can be used.
;
; DEC (nav star data) is in form 0...65535 where 65535 is 360 degrees counting from -90
; If we use 65536 for 360 deg, this is 182.044 per degree and 3.034 per minute.
; Then, counts = (dd*256 + mm) * EncMax / 65536
; This is OK if Max counts << 65536.
; We thus store dec as a 2 byte number in the range 0-65535, thus
; Converting to encoder counts reduces to dec * DECMax / 65536.


; multiply DECdd,mm by DECmax / 65536 to get DECcount

dectocount                    
        movf    DECmax, w
        movwf   temp0
        movf    DECmaxHi, w
        movwf   temp1
        movf    DECdd, w            
        movwf   fact1
        movf    DECmm, w            
        movwf   fact0
        clrf    fact2
        call    mult16          ; result in dest4,3,2   0,1 discard
        movf    dest2  , W
        movwf   DECcount
        movf    dest3, W
        movwf   DECcountHi
        return

; Navigation stars
; I put this here on 0x700 so that it fits on a single 256 byte page.
; The calling function needs to set PCLATH to 0x07 to read this table.
; ra,mm,ss,dd,mm,ss,0 = 7 bytes plus name[8],0 = 16 bytes
; Change April 2006: dec ss byte no longer needed (was N/S flag).
; 16 stars x16 = 256, last entry shortened to keep to 255 bytes.
; Reminder: these are the alphabetic codes in our lookup table...
;  a  b  c  d  e  f  g  h  i  j  k  l  m  n  o  p  q  r  s  t  u  v  w  x  y  z
; 10 11 12 13 14 15  9 25 29       17    20 22 19 26 18  5 16 27          28  2  decimal
;  A  B  C  D  E  F  9 19 1D       11    14 16 13 1A 12  5 10 1B          1C  2  hex
          ORG   0x700
navigation
          movwf    PCL          ; use HEX values, use 0x?? if value starts with a letter!
;   hh mm ss dd  mm. ss 0 n1 n2 n3 n4 n5 n6 n7 n8 0   #  ddmm 01234567 01234567   dec     count
 dt  1,25,2B,17,0x4D,80,0,0A,0C,19,0E,12,14,0A,12,0 ; 0  174D Achernar A Eridanu -57.235   5965
 dt  4,23,37,4B,0xBD,10,0,0A,15,10,0A,1B,12,1B, 5,0 ; 1  4BBD A Taurus Aldebaran  16.5083 19389
 dt  5,0E,20,3A,0x2B,80,0,12,1D, 9,0E,11,15,15,15,0 ; 2  3A2B Rigel... B Orion    -8.2016 14891
;dt  5,37,0A,45,0x44,10,0,0A,15,16,12,1D,16,14,15,0 ; 2  4544 A Orion. Betelgeus   7.4066 17732
 dt  6,17,39,1A,0x87,80,0,0C,0A,14,16,13,1B, 5,15,0 ; 3  1A87 Canopus. A Carina  -52.6950  6791
 dt  6,2D, 9,34,0x1E,80,0, 5,1D,12,1D,1B, 5,15,15,0 ; 4  341E Sirius.. A Canis Mj-16.7116 13342
 dt  7,22,24,56,0xAD,10,0,0C,0A, 5,10,16,12,15,15,0 ; 5  56AD Castor.. A Gemini   31.8883 22189
 dt  7,27,12,43,0xB7,10,0,13,12,16,0C,1C,16,14,15,0 ; 6  43B7 Procyon  A Canis Mn  5.2266 17335
 dt 0A, 8,16,48,0x82,10,0,12,0E, 9,1B,11,1B, 5,15,0 ; 7  4882 Regulus. A Leo      11.9666 18562
 dt 0C,1A,24,13,0x21,80,0,0A,15,0C,12,1B,0C,1D, 5,0 ; 8  1321 A Crucis Acrux     -63.0983  4897
 dt 0D,19,0B,38,0x10,80,0, 5,13,1D,0C,0A,15,15,15,0 ; 9  3810 Spica... A Virgo   -11.1600 14352
 dt 0E,0F,28,4D,0xA5,10,0,0A,12,0C,10,1B,12,1B, 5,0 ; 10 4DA5 Arcturus A Boot     19.1883 19877
 dt 0E,27,29,14,0xBE,80,0,0A,15,0C,0E,14,10,15,15,0 ; 11 14BE A Cent.. Rigil Ken -60.8333  5310
 dt 10,1D,18,2D,0x34,80,0,0A,14,10,0A,12,0E, 5,15,0 ; 12 2D34 Antares. A Scorpio -26.4316 11572
 dt 12,24,38,5B,0x94,10,0,0A,15,11,1C,12,0A,15,15,0 ; 13 5B94 A Lyra.. Vega       38.7816 23444
 dt 13,32,2F,46,0x4E,10,0,0A,11,10,0A,1D,12,15,15,0 ; 14 464E Altair.. A Aquila    8.8666 17998
 dt 16,39,27,2A,0xF0,80,0,0A,15,13, 5,0A, 0, 0, 0   ; 15 2AF0 A Psa..  Formalhau -29.6216 10992
; ******************************************************


; default ID codes and custom code

;           ORG     0x2000
;           DATA    0x30                ; version
;           DATA    0x30
;           DATA    0x41                ; custom id
;           DATA    0x48
 

; default EEPROM data
 
            ORG     0x2100

            DATA    0x00      ; 00 pointnum
            DATA    0x00      ; 01 Time sec
            DATA    0x1E      ; 02 min
            DATA    0x13      ; 03 hours      7:30PM = 19 30 00
            DATA    0x00      ; 04 sid sec
            DATA    0x00      ; 05 min
            DATA    0x01      ; 06 hour       01 00 00
            DATA    0x54      ; 07 HAfact0  was e4272a
            DATA    0xE9      ; 08 HAfact1
            DATA    0x03      ; 09 HAfact2
            DATA    0xEE      ; 0A DECfact0 was 138155
            DATA    0xC8      ; 0B DECfact1
            DATA    0x19      ; 0C DECfact2
            DATA    0x00      ; 0D decsign  
            DATA    0x20      ; 0E editloc
            DATA    0x00      ; 0F navigation star
            DATA    0xB9      ; 10 DEC MAX low 8377 was 9986
            DATA    0x20      ; 11
            DATA    0x49      ; 12 RA MAX  low 22089 was 21988
            DATA    0x56      ; 13
            DATA    0xF2      ; 14 RA count    /2
            DATA    0x2A      ; 15 
            DATA    0xE8      ; 16 LAT count   1803=-25deg
            DATA    0x05      ; 17
            DATA    0xC0      ; 18 DEC count   9986/4
            DATA    0x08      ; 19
            DATA    0x0C      ; 1A RAhh       12 00 00
            DATA    0x00      ; 1B RAmm       
            DATA    0x00      ; 1C RAss
            DATA    0x13      ; 1D DECdd      19 00.0
            DATA    0x00      ; 1E DECmm
            DATA    0x00      ; 1F DECss
            ; set point 0 data: 5 bytes   16 set points
            ; low value test point 
            DATA    0x0B      ; 20 DECcount   11  -89 34.1 
            DATA    0x00      ; 21 DECcountHi
            DATA    0x00      ; 22 RAhh       00 00 00
            DATA    0x00      ; 23 RAmm       
            DATA    0x00      ; 24 RAss
            ; set point 1 data: 5 bytes
            ; high value test point 
            DATA    0x80      ; 25 DECcount   
            DATA    0x13      ; 26 DECcountHi
            DATA    0x17      ; 27 RAhh       23 58 58
            DATA    0x3A      ; 28 RAmm       
            DATA    0x3A      ; 29 RAss
            ; set point 2 data: 5 bytes
            ; mid value test point 
            DATA    0xC0      ; 2A DECcount   9986/4  
            DATA    0x09      ; 2B DECcountHi
            DATA    0x0C      ; 2C RAhh       12 00 00
            DATA    0x00      ; 2D RAmm       
            DATA    0x00      ; 2E RAss
            ; set point 3 data: 5 bytes 
            DATA    0x81      ; 2F DECcount   9986-129
            DATA    0x26      ; 30 DECcountHi
            DATA    0x17      ; 31 RAhh       23 48 48
            DATA    0x30      ; 32 RAmm       
            DATA    0x30      ; 33 RAss
            ; set point 4 data: 5 bytes 
            DATA    0x81      ; 34 DECcount   9986-129
            DATA    0x26      ; 35 DECcountHi
            DATA    0x17      ; 36 RAhh       23 48 48
            DATA    0x30      ; 37 RAmm       
            DATA    0x30      ; 38 RAss
            ; set point 5 data: 5 bytes 
            DATA    0x81      ; 39 DECcount   9986-129
            DATA    0x26      ; 3A DECcountHi
            DATA    0x17      ; 3B RAhh       23 48 48
            DATA    0x30      ; 3C RAmm       
            DATA    0x30      ; 3D RAss
            ; set point 6 data: 5 bytes 
            DATA    0x81      ; 3E DECcount   9986-129
            DATA    0x26      ; 3F DECcountHi
            DATA    0x17      ; 40 RAhh       23 48 48
            DATA    0x30      ; 41 RAmm       
            DATA    0x30      ; 42 RAss
            ; set point 7 data: 5 bytes 
            DATA    0x81      ; 43 DECcount   9986-129
            DATA    0x26      ; 44 DECcountHi
            DATA    0x17      ; 45 RAhh       23 48 48
            DATA    0x30      ; 46 RAmm       
            DATA    0x30      ; 47 RAss
            ; set point data finishes here
            ; the next 48 bytes are reserved for stopwatch log 
            DATA    0x81      ; 48 DECcount   9986-129
            DATA    0x26      ; 49 DECcountHi
            DATA    0x17      ; 4A RAhh       23 48 48
            DATA    0x30      ; 4B RAmm       
            DATA    0x30      ; 4C RAss
            DATA    0x81      ; 4D DECcount   9986-129
            DATA    0x26      ; 4E DECcountHi
            DATA    0x17      ; 4F RAhh       23 48 48
            DATA    0x30      ; 50 RAmm       
            DATA    0x30      ; 51 RAss
            DATA    0x81      ; 52 DECcount   9986-129
            DATA    0x26      ; 53 DECcountHi
            DATA    0x17      ; 54 RAhh       23 48 48
            DATA    0x30      ; 55 RAmm       
            DATA    0x30      ; 56 RAss
            DATA    0x81      ; 57 DECcount   9986-129
            DATA    0x26      ; 58 DECcountHi
            DATA    0x17      ; 59 RAhh       23 48 48
            DATA    0x30      ; 5A RAmm       
            DATA    0x30      ; 5B RAss
            DATA    0x81      ; 5C DECcount   9986-129
            DATA    0x26      ; 5D DECcountHi
            DATA    0x17      ; 5E RAhh       23 48 48
            DATA    0x30      ; 5F RAmm       
            DATA    0x30      ; 60 RAss
            DATA    0x81      ; 61 DECcount   9986-129
            DATA    0x26      ; 62 DECcountHi
            DATA    0x17      ; 63 RAhh       23 48 48
            DATA    0x30      ; 64 RAmm       
            DATA    0x30      ; 65 RAss
            DATA    0x81      ; 66 DECcount   9986-129
            DATA    0x26      ; 67 DECcountHi
            DATA    0x17      ; 68 RAhh       23 48 48
            DATA    0x30      ; 69 RAmm       
            DATA    0x30      ; 6A RAss
            DATA    0x81      ; 6B DECcount   9986-129
            DATA    0x26      ; 6C DECcountHi
            DATA    0x17      ; 6D RAhh       23 48 48
            DATA    0x30      ; 6E RAmm       
            DATA    0x30      ; 6F RAss

            ORG     0x2170    ; 16 bytes, version and programers id code

            ;       "0123456789ABCDEF"
            dt      "1.02 amhugo 2007"
            
		end                       ; directive 'end of program'


